# -*- coding: UTF-8 -*-
import sys
import types
import logging
import socket
import platform


try:
    import zmq
    ZMQ_AVAILABLE = True
except ImportError:
    ZMQ_AVAILABLE = False

try:
    import msgpack
    MSGPACK_AVAILABLE = True
except ImportError:
    MSGPACK_AVAILABLE = False

try:
    import __pypy__
    IS_PYPY = True
except ImportError:
    IS_PYPY = False

# In python3 the types constants have been eliminated.
if sys.hexversion > 0x03000000:
    TYPENAMES_TO_TYPE = {'None': type(None),
                         'Boolean': bool,
                         'Bool': bool,
                         'Integer': int,
                         'Int': int,
                         'Float': float,
                         'Str': str,
                         'String': str,
                         'Unicode': str,
                         'Tuple': tuple,
                         'List': list,
                         'Dictionary': dict,
                         'Dict': dict}
else:
    TYPENAMES_TO_TYPE = {'None': types.NoneType,
                         'Boolean': types.BooleanType,
                         'Bool': types.BooleanType,
                         'Integer': types.IntType,
                         'Int': types.IntType,
                         'Float': types.FloatType,
                         'Str': types.StringType,
                         'String': types.StringType,
                         'Unicode': types.UnicodeType,
                         'Tuple': types.TupleType,
                         'List': types.ListType,
                         'Dictionary': types.DictType,
                         'Dict': types.DictType}

# loglevel_string_to_loglevel_int
LOGLEVEL_STRING_TO_LOGLEVEL_INT = {'info': logging.INFO,
                                   'warn': logging.WARN,
                                   'error': logging.ERROR,
                                   'critical': logging.CRITICAL,
                                   'fatal': logging.FATAL,
                                   'debug': logging.DEBUG}

MY_HOSTNAME = socket.gethostname()
MY_SYSTEM_NAME = platform.system()


# -*- coding: utf-8 -*-
import sys

import redis

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Buffers import Buffer
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue


@ModuleDocstringParser
class RedisListSink(BaseThreadedModule):
    """
    Send events to a redis lists.

    list: Name of redis list to send data to.
    server: Redis server to connect to.
    port: Port redis server is listening on.
    db: Redis db.
    password: Redis password.
    format: Which event fields to send on, e.g. '$(@timestamp) - $(url) - $(country_code)'. If not set the whole event dict is send.
    store_interval_in_secs: Send data to redis in x seconds intervals.
    batch_size: Send data to redis if event count is above, even if store_interval_in_secs is not reached.
    backlog_size: Maximum count of events waiting for transmission. Events above count will be dropped.

    Configuration template:

    - RedisListSink:
       list:                            # <type: String; is: required>
       server:                          # <default: 'localhost'; type: string; is: optional>
       port:                            # <default: 6379; type: integer; is: optional>
       db:                              # <default: 0; type: integer; is: optional>
       password:                        # <default: None; type: None||string; is: optional>
       format:                          # <default: None; type: None||string; is: optional>
       store_interval_in_secs:          # <default: 5; type: integer; is: optional>
       batch_size:                      # <default: 500; type: integer; is: optional>
       backlog_size:                    # <default: 500; type: integer; is: optional>
    """

    module_type = "output"
    """Set module type"""
    can_run_forked = True

    def configure(self, configuration):
         # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.format = self.getConfigurationValue('format')
        self.list = self.getConfigurationValue('list')
        self.client = redis.StrictRedis(host=self.getConfigurationValue('server'),
                                          port=self.getConfigurationValue('port'),
                                          password=self.getConfigurationValue('password'),
                                          db=self.getConfigurationValue('db'))
        try:
            self.client.ping()
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not connect to redis store at %s. Exception: %s, Error: %s." % (self.getConfigurationValue('server'),etype, evalue))
            self.lumbermill.shutDown()

    def getStartMessage(self):
        return "[%s] on %s:%s. Max buffer size: %d" % (self.list,
                                                       self.getConfigurationValue('server'),
                                                       self.getConfigurationValue('port'),
                                                       self.getConfigurationValue('backlog_size'))


    def initAfterFork(self):
        BaseThreadedModule.initAfterFork(self)
        self.buffer = Buffer(self.getConfigurationValue('batch_size'), self.storeData, self.getConfigurationValue('store_interval_in_secs'), maxsize=self.getConfigurationValue('backlog_size'))

    def storeData(self, buffered_data):
        try:
            self.client.rpush(self.list, *buffered_data)
            return True
        except:
            exc_type, exc_value, exc_tb = sys.exc_info()
            self.logger.error("Could not add event to redis list %s. Exception: %s, Error: %s." % (self.list, exc_type, exc_value))
            return False

    def handleEvent(self, event):
        if self.format:
            publish_data = mapDynamicValue(self.format, event)
        else:
            publish_data = event
        self.buffer.append(publish_data)
        yield None

    def shutDown(self):
        try:
            self.buffer.flush()
        except:
            pass
        BaseThreadedModule.shutDown(self)


# -*- coding: utf-8 -*-
import collections
import logging
import multiprocessing
import sys
import time
from cStringIO import StringIO

import pywebhdfs
from pywebhdfs.webhdfs import PyWebHdfsClient

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser, setInterval
from lumbermill.utils.DynamicValues import mapDynamicValue
from lumbermill.utils.misc import TimedFunctionManager


@ModuleDocstringParser
class WebHdfsSink(BaseThreadedModule):
    """
    Store events in hdfs via webhdfs.

    server: webhdfs/https node
    user: Username for webhdfs.
    path: Path to logfiles. String my contain any of pythons strtime directives.
    name_pattern: Filename pattern. String my conatain pythons strtime directives and event fields.
    format: Which event fields to send on, e.g. '$(@timestamp) - $(url) - $(country_code)'. If not set the whole event dict is send.
    store_interval_in_secs: Send data to webhdfs in x seconds intervals.
    batch_size: Send data to webhdfs if event count is above, even if store_interval_in_secs is not reached.
    backlog_size: Maximum count of events waiting for transmission. Events above count will be dropped.
    compress: Compress output as gzip file. For this to be effective, the chunk size should not be too small.

    Configuration template:

    - WebHdfsSink:
       server:                          # <default: 'localhost:14000'; type: string; is: optional>
       user:                            # <type: string; is: required>
       path:                            # <type: string; is: required>
       name_pattern:                    # <type: string; is: required>
       format:                          # <type: string; is: required>
       store_interval_in_secs:          # <default: 10; type: integer; is: optional>
       batch_size:                      # <default: 1000; type: integer; is: optional>
       backlog_size:                    # <default: 5000; type: integer; is: optional>
       compress:                        # <default: None; type: None||string; values: [None,'gzip','snappy']; is: optional>
    """

    module_type = "output"
    """Set module type"""
    can_run_forked = True

    def configure(self, configuration):
         # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        # Make urllib3s logging less verbose.
        urllib3_logger = logging.getLogger('requests.packages.urllib3.connectionpool')
        urllib3_logger.setLevel(logging.CRITICAL)
        self.server, self.port = self.getConfigurationValue('server').split(':')
        self.user = self.getConfigurationValue('user')
        self.events_container = []
        self.batch_size = self.getConfigurationValue('batch_size')
        self.backlog_size = self.getConfigurationValue('backlog_size')
        self.path = self.getConfigurationValue('path')
        self.name_pattern = self.getConfigurationValue('name_pattern')
        self.format = self.getConfigurationValue('format')
        self.compress = self.getConfigurationValue('compress')
        if self.compress == 'gzip':
            try:
                import gzip
            except ImportError:
                self.logger.error('Gzip compression selected but gzip module could not be loaded.')
                self.lumbermill.shutDown()
        if self.compress == 'snappy':
            try:
                import snappy
            except ImportError:
                self.logger.error('Snappy compression selected but snappy module could not be loaded.')
                self.lumbermill.shutDown()
        self.is_storing = False
        self.lock = multiprocessing.Lock()
        self.timed_store_func = self.getTimedStoreFunc()

    def getHdfsClient(self):
        try:
            hdfs = PyWebHdfsClient(host=self.server, port=self.port, user_name=self.user)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error('Could not connect to webfs service on %s. Exception: %s, Error: %s.' % (self.getConfigurationValue('server'), etype, evalue))
            return None
        return hdfs

    def getTimedStoreFunc(self):
        @setInterval(self.getConfigurationValue('store_interval_in_secs'))
        def timedStoreEvents():
            if self.is_storing or self.lock._semlock._is_mine():
                return
            self.storeEvents(self.events_container)
        return timedStoreEvents

    def ensureFileExists(self, path):
        try:
            self.hdfs.get_file_dir_status(path)
            success = True
        except pywebhdfs.errors.FileNotFound:
            success = self.createFile(path)
        return success

    def ensureDirExists(self, path):
        try:
            self.hdfs.get_file_dir_status(path)
            success = True
        except pywebhdfs.errors.FileNotFound:
            success = self.createDir(path)
        return success

    def createDir(self, path, recursive=True):
        try:
            self.hdfs.make_dir(path)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error('Could not create directory %s. Exception: %s, Error: %s.' % (path, etype, evalue))
            return False
        return True

    def createFile(self, path):
        try:
            data = ''
            if self.compress == 'snappy':
                data = self.getSnappyHeader()
            self.hdfs.create_file(path, data)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error('Could not create file %s. Exception: %s, Error: %s.' % (path, etype, evalue))
            return False
        return True

    def initAfterFork(self):
        self.hdfs = self.getHdfsClient()
        TimedFunctionManager.startTimedFunction(self.timed_store_func)
        # Call parent run method
        BaseThreadedModule.initAfterFork(self)

    def handleEvent(self, event):
        # Wait till a running store is finished to avoid strange race conditions while manipulating self.events_container.
        while self.is_storing:
            time.sleep(.0001)
        while len(self.events_container) > self.backlog_size:
            self.logger.warning("Maximum number of items (%s) in buffer reached. Waiting for flush." % self.maxsize)
            time.sleep(1)
        self.events_container.append(event)
        if len(self.events_container) == self.batch_size:
            self.storeEvents(self.events_container)
        yield None

    def storeEvents(self, events):
        """
        As a sidenote: synchronizing multiple processes with a lock to ensure only one process will write to a given
        file, seems not to work as expected. webhdfs does not directly free a lease on a file after appending.
        A better approach seems to be to retry the write a number of times before failing.
        """
        if len(events) == 0:
            return
        self.is_storing = True
        path = time.strftime(self.path)
        self.ensureDirExists(path)
        write_data = collections.defaultdict(str)
        for event in events:
            filename = time.strftime(self.getConfigurationValue('name_pattern'))
            filename = filename % event
            line = mapDynamicValue(self.format, event)
            write_data[filename] += line
        write_tries = 0
        retry_sleep_time = .4
        for filename, lines in write_data.items():
            if self.compress == 'gzip':
                filename += ".gz"
                lines = self.compressGzip(lines)
            elif self.compress == 'snappy':
                filename += ".snappy"
                lines = self.compressSnappy(lines)
            while write_tries < 10:
                try:
                    self.ensureFileExists('%s/%s' % (path, filename))
                    self.hdfs.append_file('%s/%s' % (path, filename), lines)
                    break
                except KeyError:
                    etype, evalue, etb = sys.exc_info()
                    self.logger.error('Could no log event %s. The format key %s was not present in event.' % (event, evalue))
                except pywebhdfs.errors.PyWebHdfsException:
                    write_tries += 1
                    # Retry max_retry times. This can solve problems like leases beeing hold by another process.
                    if write_tries < 10:
                        time.sleep(retry_sleep_time * write_tries)
                        continue
                    # Issue error after max retries.
                    etype, evalue, etb = sys.exc_info()
                    self.logger.error('Max write retries reached. Could no log event %s. Exception: %s, Error: %s.' % (event, etype, evalue))
        self.events_container = []
        self.is_storing = False

    def compressGzip(self, data):
        buffer = StringIO()
        compressor = gzip.GzipFile(mode='wb', fileobj=buffer)
        try:
            compressor.write(data)
        finally:
            compressor.close()
        return buffer.getvalue()

    def compressSnappy(self, data):
        compressed = snappy.compress(data)
        return compressed



# -*- coding: utf-8 -*-
import multiprocessing
from collections import defaultdict

from lumbermill.utils import Decorators


@Decorators.Singleton
class StatisticCollector:

    def __init__(self):
        self.counter_stats = defaultdict(int)
        #self.counter_stats_per_module = defaultdict(lambda: defaultdict(int))
        """ Stores the statistic data """

    def incrementCounter(self, name, increment_value=1):
        self.counter_stats[name] += increment_value
        #print "Incr %s: %s" % (name, self.counter_stats[name])

    def decrementCounter(self, name, decrement_value=1):
        self.counter_stats[name] -= decrement_value
        #print "Decr %s: %s" % (name, self.counter_stats[name])

    def resetCounter(self,name):
        self.counter_stats[name] = 0

    def setCounter(self, name, value):
        self.counter_stats[name] = value

    def getCounter(self, name):
        return self.counter_stats[name]

    def getAllCounters(self):
        return self.counter_stats

@Decorators.Singleton
class MultiProcessStatisticCollector:

    def __init__(self):
        self.lock = multiprocessing.Lock()
        self.sync_manager = multiprocessing.Manager()
        self.counter_stats = self.sync_manager.dict()
        """ Stores the statistic data """

    def incrementCounter(self, name, increment_value=1):
        with self.lock:
            try:
                self.counter_stats[name] += increment_value
            except KeyError:
                self.counter_stats[name] = increment_value
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it
                pass

    def decrementCounter(self, name, decrement_value=1):
        with self.lock:
            try:
                self.counter_stats[name] -= decrement_value
            except KeyError:
                self.counter_stats[name] = 0
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it
                pass

    def resetCounter(self,name):
        with self.lock:
            try:
                self.counter_stats[name] = 0
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it
                pass

    def setCounter(self, name, value):
        with self.lock:
            try:
                self.counter_stats[name] = value
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it
                pass

    def getCounter(self, name):
        with self.lock:
            try:
                return self.counter_stats[name]
            except KeyError:
                return 0
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it
                return 0

    def getAllCounters(self):
        return self.counter_stats

    def shutDown(self):
        self.sync_manager.shutdown()


import re
import sys
from optparse import OptionParser 

parser = OptionParser()
parser.add_option("-r", "--regex", dest="regex")
parser.add_option("-s", "--string", dest="string")
parser.add_option("-f", "--file", dest="file")

(options, args) = parser.parse_args()
if options.file:
    with open(options.file) as f:
        string = f.readlines()
    test_string = string[0]
else:
    test_string = options.string
try:
    regex_comp = re.compile(options.regex)
    matches = regex_comp.search(test_string)
    if not matches:
        print "Regex does not match."
        sys.exit(255)
    print "Regex matches:"
    print "%s" % matches.groupdict()
except Exception, e:
    print "Could not test regex. Exception: %s, Error: %s" % (Exception ,e)


# -*- coding: utf-8 -*-
import tornado.web

import handler.ActionHandler
import handler.HtmlHandler
import handler.WebsocketHandler
import uimodules.WebGui.ServerInfo
from lumbermill.BaseModule import BaseModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class WebGui(BaseModule):
    """
    A WebGui plugin for LumberMill. At the moment this is just a stub.

    Module dependencies:    WebserverTornado

    Configuration template:

    tornado_webserver: Name of the webserver module.
    document_root: Path to look for templates and static files.

    - WebGui:
       tornado_webserver: webserver     # <default: 'WebserverTornado'; type: string; is: optional>
       document_root: other_root        # <default: 'docroot'; type: string; is: optional>
    """
    module_type = "stand_alone"
    """Set module type"""

    can_run_forked = False

    def configure(self, configuration):
        # Call parent configure method
        BaseModule.configure(self, configuration)
        # Get tornado webserver module instance.
        mod_info = self.lumbermill.getModuleInfoById(self.getConfigurationValue('tornado_webserver'))
        if not mod_info:
            self.logger.error("Could not start web gui module. Required webserver module %s not found. Please check your configuration." % (self.getConfigurationValue('tornado_webserver')))
            self.lumbermill.shutDown()
            return
        self.webserver_module = mod_info['instances'][0]
        self.webserver_module.addUiModules([uimodules.WebGui.ServerInfo])
        self.webserver_module.addHandlers(self.getHandlers())

    def getHandlers(self):
        settings = self.webserver_module.getSettings()
        handlers =  [# HtmlHandler
                     (r"/", handler.HtmlHandler.MainHandler),
                     (r"/server_configuration", handler.HtmlHandler.ServerConfigurationAsText),
                     (r"/configuration", handler.HtmlHandler.ConfigurationHandler),
                     # StaticFilesHandler
                     (r"/images/(.*)",tornado.web.StaticFileHandler, {"path": "%s/images/" % settings['static_path']}),
                     (r"/css/(.*)",tornado.web.StaticFileHandler, {"path": "%s/css/" % settings['static_path']}),
                     (r"/js/(.*)",tornado.web.StaticFileHandler, {"path": "%s/js/" % settings['static_path']},),
                     # REST ActionHandler
                     (r"/rest/server/restart", handler.ActionHandler.RestartHandler),
                     (r"/rest/server/info", handler.ActionHandler.GetServerInformation),
                     (r"/rest/server/configuration", handler.ActionHandler.GetServerConfiguration),
                     # WebsocketHandler
                     (r"/websockets/statistics", handler.WebsocketHandler.StatisticsWebSocketHandler),
                     (r"/websockets/get_logs", handler.WebsocketHandler.LogToWebSocketHandler)]
        return handlers


# -*- coding: utf-8 -*-
import time

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class Tarpit(BaseThreadedModule):
    """
    Send an event into a tarpit before passing it on.

    Useful only for testing purposes of threading problems and concurrent access to event data.

    Configuration template:

    - Tarpit:
       delay:                           # <default: 10; type: integer; is: optional>
       receivers:
        - NextModule
    """

    module_type = "misc"
    """Set module type"""

    def handleEvent(self, event):
        """
        Process the event.

        @param event: dictionary
        @return data: dictionary
        """
        time.sleep(self.getConfigurationValue('delay'))
        yield event


# -*- coding: utf-8 -*-
import tornado.web
import tornado.escape
import tornado.websocket
import tornado.gen
import logging
import time


class WebsocketLoggingHandler(logging.Handler):
    """
    A handler class which allows to log to a websocket.
    """

    def __init__(self, websocket_handler):
        logging.Handler.__init__(self)
        self.websocket_handler = websocket_handler
        self.formatter = None

    def emit(self, record):
        try:
            msg = self.format(record)
            self.websocket_handler.write_message(tornado.escape.json_encode({'timestamp': time.time(),
                                                                             'log_message': msg}))
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

class BaseHandler(tornado.web.RequestHandler):
    @property
    def webserver_module(self):
        return self.settings['TornadoWebserver']

    def __get_current_user(self):
        user_json = self.get_secure_cookie("gambolputty_web")
        if not user_json: return None
        return tornado.escape.json_decode(user_json)

class LogToWebSocketHandler(tornado.websocket.WebSocketHandler, BaseHandler):
    def open(self):
        # Create new log handler.
        self.ws_stream_handler = WebsocketLoggingHandler(self)
        # Get all configured modules and register our log handler.
        for module_name, module_info in sorted(self.webserver_module.gp.modules.items(), key=lambda x: x[1]['idx']):
            for instance in module_info['instances']:
                instance.logger.addHandler(self.ws_stream_handler)

    def on_close(self):
        # Get all configured modules and unregister our log handler.
        for module_name, module_info in sorted(self.webserver_module.gp.modules.items(), key=lambda x: x[1]['idx']):
            for instance in module_info['instances']:
                instance.logger.removeHandler(self.ws_stream_handler)

class StatisticsWebSocketHandler(tornado.websocket.WebSocketHandler, BaseHandler):
    """
    Redirect the output of the statistics module to a websocket.
    This will only work, if the statistic module is configured for the running LumberMill.
    """
    def open(self):
        # Try to get the statistics module
        statistic_module_info = self.webserver_module.gp.getModuleInfoById('Statistics')
        if not statistic_module_info:
            self.write_message(tornado.escape.json_encode(False))
            return
        self.statistic_module = statistic_module_info['instances'][0]
        self.statistic_module.registerTimedFunction(id(self), self.sendIntervalStatistics)

    def on_message(self, message):
        pass

    def sendIntervalStatistics(self):
        if self.statistic_module.getConfigurationValue('waiting_event_statistics'):
            self.receiveRateStatistics()
        if self.statistic_module.getConfigurationValue('waiting_event_statistics'):
            self.eventsInQueuesStatistics()
        if self.statistic_module.getConfigurationValue('event_type_statistics'):
            self.eventTypeStatistics()

    def receiveRateStatistics(self):
        eps = self.statistic_module.stats_collector.getCounter('eps')
        if not eps:
            eps = 0
        self.write_message(tornado.escape.json_encode({'timestamp': time.time(),
                                                       'eps': eps}))

    def eventTypeStatistics(self):
        for event_type, count in sorted(self.statistic_module.stats_collector.getAllCounters().items()):
            if not event_type.startswith('event_type_'):
                continue
            event_type = event_type.replace('event_type_', '')
            self.write_message(tornado.escape.json_encode({'timestamp': time.time(),
                                                           'event_type': event_type,
                                                           'count': count}))

    def eventsInQueuesStatistics(self):
        if len(self.statistic_module.module_queues) == 0:
            return
        for module_name, queue in self.statistic_module.module_queues.items():
            self.write_message(tornado.escape.json_encode({'timestamp': time.time(),
                                                            'module_name': module_name,
                                                            'queue_size': queue.qsize()}))

    def on_close(self):
        self.statistic_module.unregisterTimedFunction(id(self))


# -*- coding: utf-8 -*-
import socket
import os
import psutil
import collections
import subprocess
import tornado.web
import tornado.escape
import tornado.auth
import tornado.gen

class BaseHandler(tornado.web.RequestHandler):
    @property
    def webserver_module(self):
        return self.settings['TornadoWebserver']

    def set_default_headers(self):
        # If we are a pack follower, allow leader to access these handlers.
        cluster_info = self.webserver_module.gp.getModuleInfoById('Cluster')
        if cluster_info:
            cluster_module = cluster_info['instances'][0]
            if cluster_module.leader:
                return
            self.set_header("Access-Control-Allow-Origin", "http://%s:%s" % (cluster_module.getDiscoveredLeader().getHostName(), cluster_module.getDiscoveredLeader().getPort()))
            self.set_header("Access-Control-Allow-Methods", "GET,PUT,POST,DELETE,OPTIONS")
            self.set_header("Access-Control-Allow-Headers","Content-Type, Depth, User-Agent, X-File-Size, X-Requested-With,\
                                                            X-Requested-By, If-Modified-Since, X-File-Name, Cache-Control")

    def __get_current_user(self):
        user_json = self.get_secure_cookie("gambolputty_web")
        if not user_json: return None
        return tornado.escape.json_decode(user_json)

class GetServerInformation(BaseHandler):
    def get(self):
        mem = psutil.virtual_memory()
        partitions = psutil.disk_partitions()
        disk_usage = {}
        for partition in partitions:
            try:
                # When running with pypy, check for missing os.statvfs module. (https://bugs.pypy.org/issue833)
                os.statvfs
                du = psutil.disk_usage(partition.mountpoint)
                disk_usage[partition.mountpoint] = {'total': du.total, 'used': du.used, 'free': du.free, 'percent': du.percent}
            except AttributeError:
                # Use fallback.
                df = subprocess.Popen(["df", partition.mountpoint], stdout=subprocess.PIPE)
                output = df.communicate()[0]
                device, size, used, available, percent, mountpoint = output.split("\n")[1].split()
                disk_usage[partition.mountpoint] = {'total': int(size)*1024, 'used': int(used)*1024, 'free': int(available)*1024, 'percent': percent}
        try:
            cpu_count = psutil.NUM_CPUS
        except AttributeError:
            cpu_count = psutil.cpu_count()
        self.write(tornado.escape.json_encode({ 'hostname': socket.gethostname(),
                                                'cpu_count': cpu_count,
                                                'load': os.getloadavg(),
                                                'memory': {'total': mem.total, 'used': mem.used, 'available': mem.available, 'percent': mem.percent},
                                                'disk_usage': disk_usage,
                                                'configuration': self.webserver_module.gp.configuration}))

class GetServerConfiguration(BaseHandler):
    def get(self):
        modules_info = collections.OrderedDict()
        for module_id, module_info in sorted(self.webserver_module.gp.modules.items(), key=lambda x: x[1]['idx']):
            modules_info[module_id] = {'id': module_id, 'type': module_info['type'], 'configuration': module_info['configuration']}
        self.write(tornado.escape.json_encode(modules_info))

class RestartHandler(BaseHandler):
    def get(self):
        self.add_header('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0')
        self.write(tornado.escape.json_encode({'restart': True}))
        self.flush()
        self.webserver_module.gp.restart()

class AuthLoginHandler(BaseHandler, tornado.auth.GoogleMixin):
    @tornado.gen.coroutine
    def get(self):
        if self.get_argument("openid.mode", None):
            user = yield self.get_authenticated_user()
            self.set_secure_cookie("gambolputty_web",tornado.escape.json_encode(user))
            self.redirect("/")
            return
        self.authenticate_redirect(ax_attrs=["name"])

class AuthLogoutHandler(BaseHandler):
    def get(self):
        self.clear_cookie("gambolputty_web")
        self.write("You are now logged out")


# -*- coding: utf-8 -*-
import logging
import sys
import time
import types
import requests
from elasticsearch import Elasticsearch, connection
from ctypes import c_char_p
from multiprocessing import Manager, Lock

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.constants import IS_PYPY
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue

# For pypy the default json module is the fastest.
if IS_PYPY:
    import json
else:
    json = False
    for module_name in ['ujson', 'yajl', 'simplejson', 'json']:
        try:
            json = __import__(module_name)
            break
        except ImportError:
            pass
    if not json:
        raise ImportError


@ModuleDocstringParser
class ElasticSearch(BaseThreadedModule):
    """
    Get documents from ElasticSearch.

    The elasticsearch module takes care of discovering all nodes of the elasticsearch cluster.
    Requests will the be loadbalanced via round robin.

    query:              The query to be executed, in json format.
    search_type:        The default search type just will return all found documents. If set to 'scan' it will return
                        'batch_size' number of found documents, emit these as new events and then continue until all
                        documents have been sent.
    field_mappings:     Which fields from the result document to add to the new event.
                        If set to 'all' the whole document will be sent unchanged.
                        If a list is provided, these fields will be copied to the new event with the same field name.
                        If a dictionary is provided, these fields will be copied to the new event with a new field name.
                        E.g. if you want "_source.data" to be copied into the events "data" field, use a mapping like:
                        "{'_source.data': 'data'}.
                        For nested values use the dot syntax as described in:
                        http://lumbermill.readthedocs.org/en/latest/introduction.html#event-field-notation
    nodes:              Configures the elasticsearch nodes.
    connection_type:    One of: 'thrift', 'http'.
    http_auth:          'user:password'.
    use_ssl:            One of: True, False.
    index_name:         Sets the index name. Timepatterns like %Y.%m.%d are allowed here.
    sniff_on_start:     The client can be configured to inspect the cluster state to get a list of nodes upon startup.
                        Might cause problems on hosts with multiple interfaces. If connections fail, try to deactivate this.
    sniff_on_connection_fail: The client can be configured to inspect the cluster state to get a list of nodes upon failure.
                              Might cause problems on hosts with multiple interfaces. If connections fail, try to deactivate this.
    query_interval_in_secs:   Get data to es in x seconds intervals. NOT YET IMPLEMENTED!!

    Configuration template:

    - ElasticSearch:
       query:                           # <default: '{"query": {"match_all": {}}}'; type: string; is: optional>
       search_type:                     # <default: 'normal'; type: string; is: optional; values: ['normal', 'scan']>
       batch_size:                      # <default: 1000; type: integer; is: optional>
       field_mappings:                  # <default: 'all'; type: string||list||dict; is: optional;>
       nodes:                           # <type: string||list; is: required>
       connection_type:                 # <default: 'urllib3'; type: string; values: ['urllib3', 'requests']; is: optional>
       http_auth:                       # <default: None; type: None||string; is: optional>
       use_ssl:                         # <default: False; type: boolean; is: optional>
       index_name:                      # <default: 'lumbermill-%Y.%m.%d'; type: string; is: optional>
       sniff_on_start:                  # <default: True; type: boolean; is: optional>
       sniff_on_connection_fail:        # <default: True; type: boolean; is: optional>
       query_interval_in_secs:          # <default: 5; type: integer; is: optional>
       receivers:
        - NextModule
    """

    module_type = "input"
    """Set module type"""
    can_run_forked = False

    def configure(self, configuration):
        # Call parent configure method.
        BaseThreadedModule.configure(self, configuration)
        # Set log level for elasticsarch library if configured to other than default.
        if self.getConfigurationValue('log_level') != 'info':
            logging.getLogger('elasticsearch').setLevel(self.logger.level)
            logging.getLogger('requests').setLevel(self.logger.level)
        else:
            logging.getLogger('elasticsearch').setLevel(logging.WARN)
            logging.getLogger('requests').setLevel(logging.WARN)
        self.query = self.getConfigurationValue('query')
        # Test if query is valid json.
        try:
            json.loads(self.query)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Parsing json query %s failed. Exception: %s, Error: %s." % (self.query, etype, evalue))
            self.lumbermill.shutDown()
        self.search_type = self.getConfigurationValue('search_type')
        self.batch_size = self.getConfigurationValue('batch_size')
        self.field_mappings = self.getConfigurationValue('field_mappings')
        self.es_nodes = self.getConfigurationValue('nodes')
        if not isinstance(self.es_nodes, list):
            self.es_nodes = [self.es_nodes]
        self.index_name_pattern = self.getConfigurationValue('index_name')
        self.index_name = mapDynamicValue(self.index_name_pattern, use_strftime=True).lower()
        if self.getConfigurationValue("connection_type") == 'urllib3':
            self.connection_class = connection.Urllib3HttpConnection
        elif self.getConfigurationValue('connection_type') == 'requests':
            self.connection_class = connection.RequestsHttpConnection
        self.lock = Lock()
        self.manager = Manager()
        if self.search_type == 'scan':
            self.can_run_forked = True
            scroll_id = self.getInitalialScrollId()
            if not scroll_id:
                self.lumbermill.shutDown()
            self.shared_scroll_id = self.manager.Value(c_char_p, scroll_id)
        elif self.search_type == 'normal':
            self.query_from = 0
            self.query = json.loads(self.query)
            self.query['size'] = self.batch_size

    def getInitalialScrollId(self):
        scroll_id = None
        try:
            response = requests.get('http://%s/%s/_search?search_type=scan&scroll=1m&size=%s' % (self.es_nodes[0], self.index_name, self.batch_size), data=self.query)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not initialize scan search. Exception: %s, Error: %s." % (etype, evalue))
        try:
            results = json.loads(response.text)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not parse query response. Exception: %s, Error: %s." % (etype, evalue))
        if '_scroll_id' in results:
            scroll_id = results['_scroll_id']
        else:
            self.logger.error("Could not get initial scroll id. Response: %s." % results)
        return scroll_id

    def initAfterFork(self):
        BaseThreadedModule.initAfterFork(self)
        if self.search_type == 'scan':
            self.simple_es_client = requests.Session()
        elif self.search_type == 'normal':
            self.es = self.connect()

    def connect(self):
        es = False
        tries = 0
        while tries < 5 and not es:
            try:
                # Connect to es node and round-robin between them.
                self.logger.debug("Connecting to %s." % self.es_nodes)
                es = Elasticsearch(self.es_nodes,
                                     connection_class=self.connection_class,
                                     sniff_on_start=self.getConfigurationValue('sniff_on_start'),
                                     sniff_on_connection_fail=self.getConfigurationValue('sniff_on_connection_fail'),
                                     sniff_timeout=5,
                                     maxsize=20,
                                     use_ssl=self.getConfigurationValue('use_ssl'),
                                     http_auth=self.getConfigurationValue('http_auth'))
            except:
                etype, evalue, etb = sys.exc_info()
                self.logger.warning("Connection to %s failed. Exception: %s, Error: %s." % (self.es_nodes,  etype, evalue))
                self.logger.warning("Waiting %s seconds before retring to connect." % ((4 + tries)))
                time.sleep(4 + tries)
                tries += 1
                continue
        if not es:
            self.logger.error("Connection to %s failed. Shutting down." % self.es_nodes)
            self.lumbermill.shutDown()
        else:
            self.logger.debug("Connection to %s successful." % self.es_nodes)
        return es

    def run(self):
        while self.alive:
            if self.search_type == 'scan':
                found_documents = self.executeScrollQuery()
            elif self.search_type == 'normal':
                found_documents = self.executeQuery()
            if not found_documents:
                self.alive = False
                break
            for doc in found_documents:
                # No special fields were selected.
                # Merge _source field and all other elasticsearch fields to one level.
                doc.update(doc.pop('_source'))
                if isinstance(self.field_mappings, types.ListType):
                    doc = self.extractFieldsFromResultDocument(self.field_mappings, doc)
                elif isinstance(self.field_mappings, types.DictType):
                    doc = self.extractFieldsFromResultDocumentWithMapping(self.field_mappings, doc)
                event = DictUtils.getDefaultEventDict(dict=doc, caller_class_name=self.__class__.__name__)
                self.sendEvent(event)
        self.lumbermill.shutDown()

    def executeScrollQuery(self):
        """ We do not use the elasticsearch client here, since we want to exploit all cores when running multiporcessed"""
        with self.lock:
            try:
               scroll_id = self.shared_scroll_id.value
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it.
                return []
            try:
                response = self.simple_es_client.get('http://%s/_search/scroll?scroll=1m' % self.es_nodes[0], data=scroll_id)
                result = json.loads(response.text)
            except:
                etype, evalue, etb = sys.exc_info()
                self.logger.error("Elasticsearch scroll query failed. Exception: %s, Error: %s." % (etype, evalue))
                return []
            # When no more hits are returned, we have processed all matching documents.
            # If an error was returned also exit.
            if not 'hits' in result or 'error' in result:
                if 'error' in result:
                    self.logger.error('Elasticsearch scroll query returned an error: %s.' % (result['error']))
                return []
            try:
                self.shared_scroll_id.value = result['_scroll_id']
            except OSError:
                # OSError: [Errno 32] Broken pipe may be thrown when exiting lumbermill via CTRL+C. Ignore it.
                pass
        return result['hits']['hits']

    def executeQuery(self):
        self.query['from'] = self.query_from
        self.query_from = self.query_from + self.batch_size
        found_documents = []
        try:
            found_documents = self.es.search(index=self.index_name, body=self.query)['hits']['hits']
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.warning("Elasticsearch query %s failed. Exception: %s, Error: %s." % (self.query, etype, evalue))
        return found_documents

    def extractFieldsFromResultDocument(self, fields, document):
        document = DictUtils.KeyDotNotationDict(document)
        new_document = DictUtils.KeyDotNotationDict()
        for field in fields:
            if field not in document:
                continue
            new_document[field] = document[field]
        return new_document

    def extractFieldsFromResultDocumentWithMapping(self, field_mapping, document):
        document = DictUtils.KeyDotNotationDict(document)
        new_document = DictUtils.KeyDotNotationDict()
        for source_field, target_field in field_mapping.iteritems():
            if source_field not in document:
                continue
            new_document[target_field] = document[source_field]
        return new_document


# -*- coding: utf-8 -*-
import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser, setInterval
from lumbermill.utils.StatisticCollector import StatisticCollector
from lumbermill.utils.misc import TimedFunctionManager


@ModuleDocstringParser
class Statistics(BaseThreadedModule):
    """
    Collect and log statistic data.

    This module keeps track of the number of times a field occured in an event during interval.
    So, if you want to count the http_status codes encountered during the last 10s, you would use this configuration:
    - Statistics:
        interval: 10
        fields: [http_status]

    After interval seconds, an event will be emitted with the following fields (counters are just examples ;):
    {'data': '',
     'event_type': 'statistic',
     'field_name': 'http_status',
     'field_counts': {'200': 5, '301': 10, '400': 5},
     'lumbermill': {'event_id': 'cef34d298fbe8ce4b662251e17b2acfb',
                     'event_type': 'statistic',
                     'received_from': False,
                     'source_module': 'Statistics'}
     'interval': 10,
     'total_count': 20}

    Configuration template:

    - Statistics:
       interval:                        # <default: 10; type: integer; is: optional>
       fields:                          # <default: ['lumbermill.event_type']; type: list; is: optional>
    """

    module_type = "misc"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.interval = self.getConfigurationValue('interval')
        self.fields = self.getConfigurationValue('fields')
        self.stats_collector = StatisticCollector()
        self.module_queues = {}

    def getRunTimedFunctionsFunc(self):
        @setInterval(self.interval)
        def runTimedFunctionsFunc():
            self.printIntervalStatistics()
        return runTimedFunctionsFunc

    def printIntervalStatistics(self):
        last_field_name = None
        field_counts = {}
        total_count = 0
        for field_name_value, field_count in sorted(self.stats_collector.getAllCounters().items()):
            if not isinstance(field_name_value, tuple):
                continue
            field_name, field_value = field_name_value
            if field_name not in self.fields:
                continue
            self.stats_collector.resetCounter(field_name_value)
            if not last_field_name:
                last_field_name = field_name
            if field_name != last_field_name:
                self.sendEvent(DictUtils.getDefaultEventDict({"total_count": total_count,  "field_name": last_field_name, "field_counts": field_counts, "interval": self.interval}, caller_class_name="Statistics", event_type="statistic"))
                last_field_name = field_name
                field_counts = {}
                total_count = 0
            field_counts.update({field_value: field_count})
            total_count += field_count
        # Send remaining.
        if last_field_name:
            self.sendEvent(DictUtils.getDefaultEventDict({"total_count": total_count, "field_name": field_name, "field_counts": field_counts, "interval": self.interval}, caller_class_name="Statistics", event_type="statistic"))

    def initAfterFork(self):
        timed_func = self.getRunTimedFunctionsFunc()
        TimedFunctionManager.startTimedFunction(timed_func)
        BaseThreadedModule.initAfterFork(self)

    def handleEvent(self, event):
        for field in self.fields:
            if field not in event:
                continue
            self.stats_collector.incrementCounter((field, event[field]))
        yield event



#coding:utf-8
import os
import socket
import sys

from tornado import netutil
from tornado.tcpserver import TCPServer

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


class SocketServer(TCPServer):

    def __init__(self, io_loop=None, ssl_options=None, gp_module=False, **kwargs):
        TCPServer.__init__(self, io_loop=io_loop, ssl_options=ssl_options, **kwargs)
        self.gp_module = gp_module
        self.address = socket.gethostname()

    def handle_stream(self, stream, address):
        ConnectionHandler(stream, self.address, self.gp_module)

class ConnectionHandler(object):

    stream_set = set([])

    def __init__(self, stream, address, gp_module):
        self.gp_module = gp_module
        self.is_open = True
        self.stream = stream
        self.address = address
        self.stream.set_close_callback(self._on_close)
        if not self.stream.closed():
            self.stream.read_until_regex(b'\r?\n', self._on_read_line)

    def _on_read_line(self, data):
        self.gp_module.sendEvent(DictUtils.getDefaultEventDict({"data": data}, caller_class_name='UnixSocket', received_from=self.address))
        if not self.stream.closed():
            self.stream.read_until_regex(b'\r?\n', self._on_read_line)

    def _on_close(self):
        self.stream.close()

@ModuleDocstringParser
class UnixSocket(BaseThreadedModule):
    """
    Reads data from an unix socket and sends it to its output queues.

    Configuration template:

    - UnixSocket:
       path_to_socket:                  # <type: string; is: required>
       receivers:
        - NextModule
    """

    module_type = "input"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.running = False

    def start(self):
        if not self.receivers:
            self.logger.error("Will not start module %s since no receivers are set." % (self.__class__.__name__))
            return
        try:
            self.unix_socket = netutil.bind_unix_socket(self.getConfigurationValue('path_to_socket'))
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Will not start module %s. Could not create unix socket %s. Exception: %s, Error: %s." % (self.__class__.__name__, self.getConfigurationValue('path_to_socket'), etype, evalue))
            return
        try:
            self.server = SocketServer(gp_module=self)
            self.server.add_socket(self.unix_socket)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not access socket %s. Exception: %s, Error: %s" % (self.getConfigurationValue("path_to_socket"), etype, evalue))
            return
        self.running = True
        #self.server.start(0)
        #ioloop.IOLoop.instance().start()

    def shutDown(self):
        if self.running:
            try:
                os.remove(self.getConfigurationValue('path_to_socket'))
            except:
                etype, evalue, etb = sys.exc_info()
                self.logger.error("Could not remove socket %s. Exception: %s, Error: %s" % (self.getConfigurationValue("path_to_socket"), etype, evalue))
        return


# -*- coding: utf-8 -*-
import os
import multiprocessing
import signal
import BaseModule


class BaseMultiProcessModule(BaseModule.BaseModule, multiprocessing.Process): #
    """
    Base class for all lumbermill modules that will run as separate processes.
    If you happen to override one of the methods defined here, be sure to know what you
    are doing ;) You have been warned...

    id: Id of the module. If multiple instances of the same modules are used, id can be used to reference the correct receiver.
    filter: Filter expression to apply to incoming events. If filter succeeds, module will handle the event, else
            the event will be passed to next module unchanged.
    pool_size: How many processes should be spawned.
    queue_size: How many events may be waiting in queue.
    queue_buffer_size: How many events will be buffered before sending them to queue.

    Configuration template:

    - module: SomeModuleName
       id:                               # <default: ""; type: string; is: optional>
       filter:                           # <default: None; type: None||string; is: optional>
       add_fields:                       # <default: {}; type: dict; is: optional>
       delete_fields:                    # <default: []; type: list; is: optional>
       event_type:                       # <default: None; type: None||string; is: optional>
       log_level:                        # <default: 'info'; type: string; values: ['info', 'warn', 'error', 'critical', 'fatal', 'debug']; is: optional>
       pool_size:                        # <default: 2; type: integer; is: optional>
       queue_size:                       # <default: 50; type: integer; is: optional>
       queue_buffer_size:                # <default: 250; type: integer; is: optional>
       receivers:
        - ModuleName
        - ModuleAlias
    """

    def __init__(self, gp):
        BaseModule.BaseModule.__init__(self, gp)
        multiprocessing.Process.__init__(self)
        self.input_queue = False
        self.alive = False
        self.worker = None

    def setInputQueue(self, queue):
        self.input_queue = queue

    def getInputQueue(self):
        return self.input_queue

    def pollQueue(self, block=True, timeout=None):
        try:
            for event in self.input_queue.get(block, timeout):
                yield event
        except (KeyboardInterrupt, SystemExit, ValueError, OSError):
            # Keyboard interrupt is catched in GambolPuttys main run method.
            # This will take care to shutdown all running modules.
            pass

    def run(self):
        if not self.receivers:
            # Only issue warning for those modules that are expected to have receivers.
            # TODO: A better solution should be implemented...
            if self.module_type not in ['stand_alone', 'output']:
                self.logger.error("Shutting down module %s since no receivers are set." % (self.__class__.__name__))
                return False
        if not self.input_queue:
            # Only issue warning for those modules that are expected to have an input queue.
            # TODO: A better solution should be implemented...
            if self.module_type not in ['stand_alone', 'input']:
                self.logger.error("Shutting down module %s since no input queue set." % (self.__class__.__name__))
                return False
        self.alive = True
        self.process_id = os.getpid()
        while self.alive:
            for event in self.pollQueue():
                if not event:
                    continue
                self.receiveEvent(event)

    def shutDown(self):
        # Call parent shutDown method
        BaseModule.BaseModule.shutDown(self)
        try:
            self.input_queue.close()
        except:
            pass
        # Kill self via signal. Otherwise a simple reload will not terminate the worker processes.
        # Why that is escapes me...
        self.alive = False
        if self.pid:
            os.kill(self.pid, signal.SIGQUIT)



# -*- coding: utf-8 -*-
import logging
import os
import random
import sys
import boto3

from lumbermill.constants import IS_PYPY
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Buffers import Buffer
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue

# For pypy the default json module is the fastest.
if IS_PYPY:
    import json
else:
    json = False
    for module_name in ['ujson', 'yajl', 'simplejson', 'json']:
        try:
            json = __import__(module_name)
            break
        except ImportError:
            pass
    if not json:
        raise ImportError


@ModuleDocstringParser
class SQSSink(BaseThreadedModule):
    """
    Send messages to amazon sqs service.

    aws_access_key_id: Your AWS id.
    aws_secret_access_key: Your AWS password.
    region: The region in which to find your sqs service.
    queue: Queue name.
    format: Which event fields to send on, e.g. '$(@timestamp) - $(url) - $(country_code)'.
            If not set event.data will be send es MessageBody, all other fields will be send as MessageAttributes.
    store_interval_in_secs: Send data to redis in x seconds intervals.
    batch_size: Number of messages to collect before starting to send messages to sqs. This refers to the internal
                receive buffer of this plugin. When the receive buffer is maxed out, this plugin will always send
                the maximum of 10 messages in one send_message_batch call.
    backlog_size: Maximum count of events waiting for transmission. Events above count will be dropped.

    values: ['us-east-1', 'us-west-1', 'us-west-2', 'eu-central-1', 'eu-west-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', 'sa-east-1', 'us-gov-west-1', 'cn-north-1']

    Configuration template:

    - SQSSink:
       aws_access_key_id:               # <type: string; is: required>
       aws_secret_access_key:           # <type: string; is: required>
       region:                          # <type: string; is: required>
       queue:                           # <type: string; is: required>
       format:                          # <default: None; type: None||string; is: optional>
       store_interval_in_secs:          # <default: 5; type: integer; is: optional>
       batch_size:                      # <default: 500; type: integer; is: optional>
       backlog_size:                    # <default: 500; type: integer; is: optional>
       receivers:
        - NextModule
    """

    module_type = "output"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        # Set boto log level.
        logging.getLogger('boto3').setLevel(logging.CRITICAL)
        logging.getLogger('botocore').setLevel(logging.CRITICAL)
        self.batch_size = self.getConfigurationValue('batch_size')
        self.format = self.getConfigurationValue('format')

    def getStartMessage(self):
        return "Queue: %s [%s]. Max buffer size: %d" % (self.getConfigurationValue('queue'),
                                                        self.getConfigurationValue('region'),
                                                        self.getConfigurationValue('backlog_size'))


    def initAfterFork(self):
        BaseThreadedModule.initAfterFork(self)
        self.buffer = Buffer(self.getConfigurationValue('batch_size'), self.storeData, self.getConfigurationValue('store_interval_in_secs'), maxsize=self.getConfigurationValue('backlog_size'))
        try:
            self.sqs_resource = boto3.resource('sqs',
                                                region_name=self.getConfigurationValue('region'),
                                                api_version=None,
                                                use_ssl=True,
                                                verify=None,
                                                endpoint_url=None,
                                                aws_access_key_id=self.getConfigurationValue('aws_access_key_id'),
                                                aws_secret_access_key=self.getConfigurationValue('aws_secret_access_key'),
                                                aws_session_token=None,
                                                config=None)
            self.sqs_queue = self.sqs_resource.get_queue_by_name(QueueName=self.getConfigurationValue('queue'))
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not connect to sqs service. Exception: %s, Error: %s." % (etype, evalue))
            self.lumbermill.shutDown()

    def handleEvent(self, event):
        self.buffer.append(event)
        yield None

    def storeData(self, buffered_data):
        batch_messages = []
        for event in buffered_data:
            try:
                id = event['lumbermill']['event_id']
            except KeyError:
                id = "%032x%s" % (random.getrandbits(128), os.getpid())
            message = {'Id': id}
            if self.format:
                event = mapDynamicValue(self.format, event)
            else:
                try:
                    event = json.dumps(event)
                except:
                    etype, evalue, etb = sys.exc_info()
                    self.logger.warning("Error while encoding event data: %s to json. Exception: %s, Error: %s." % (event, etype, evalue))
            message['MessageBody'] = event
            batch_messages.append(message)
            if len(batch_messages) % 10:
                self.sqs_queue.send_messages(Entries=batch_messages)
                batch_messages = []
        if len(batch_messages) > 0:
            self.send()

    def shutDown(self):
        self.buffer.flush()




# -*- coding: utf-8 -*-
import socket
import sys

import pygeoip

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser, memoize


@ModuleDocstringParser
class AddGeoInfo(BaseThreadedModule):
    """
    Add country_code and longitude-latitude fields based  on a geoip lookup for a given ip address.

    Here an example of fields that the module provides:
    {'city': 'Hanover', 'region_name': '06', 'area_code': 0, 'time_zone': 'Europe/Berlin', 'dma_code': 0, 'metro_code': None, 'country_code3': 'DEU', 'latitude': 52.36670000000001, 'postal_code': '', 'longitude': 9.716700000000003, 'country_code': 'DE', 'country_name': 'Germany', 'continent': 'EU'}

    geoip_dat_path: path to maxmind geoip database file.
    source_fields: list of fields to use for lookup. The first list entry that produces a hit is used.
    target: field to populate with the geoip data. If none is provided, the field will be added directly to the event.
    geo_info_fields: fields to add. Available field names:
     - area_code
     - city
     - continent
     - country_code
     - country_code3
     - country_name
     - dma_code
     - metro_code
     - postal_code
     - region_name
     - time_zone
     - latitude
     - longitude

    Configuration template:

    - AddGeoInfo:
       geoip_dat_path:                  # <type: string; is: required>
       geo_info_fields:                 # <default: None; type: None||list; is: optional>
       source_fields:                   # <default: ["x_forwarded_for", "remote_ip"]; type: list; is: optional>
       target_field:                    # <default: "geo_info"; type: string; is: optional>
       receivers:
        - NextModule
    """

    module_type = "modifier"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.gi = False
        try:
            self.gi = pygeoip.GeoIP(configuration['geoip_dat_path'], pygeoip.MEMORY_CACHE)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not init %s. Exception: %s, Error: %s. " % (self.__class__.__name__, etype, evalue))
        if not self.gi:
            self.lumbermill.shutDown()
            return False
        self.geo_info_fields = self.getConfigurationValue('geo_info_fields')
        self.source_fields = self.getConfigurationValue('source_fields')
        self.target_field = self.getConfigurationValue('target_field')

    def getStartMessage(self):
        """
        Return the module name.
        """
        return "from %s to %s" % (self.source_fields, self.target_field)


    def handleEvent(self, event):
        for lookup_field_name in self.source_fields:
            try:
                lookup_field_name_value = event[lookup_field_name]
            except KeyError:
                continue
            if not lookup_field_name_value or lookup_field_name_value == "-":
                continue
            if isinstance(lookup_field_name_value, str):
                geo_info_fields = self.getGeoIpInfo(lookup_field_name_value)
            elif isinstance(lookup_field_name_value, list):
                for field_value in lookup_field_name_value:
                    geo_info_fields = self.getGeoIpInfo(field_value)
                    if geo_info_fields:
                        break
            if geo_info_fields:
                if self.target_field:
                    event[self.target_field] = geo_info_fields
                else:
                    event.update(geo_info_fields)
                break
        yield event

    @memoize
    def getGeoIpInfo(self, hostname_or_ip):
        all_geo_info_fields = {}
        if self.is_valid_ipv4_address(hostname_or_ip) or self.is_valid_ipv6_address(hostname_or_ip):
            lookup_type = "ip_address"
        else:
            lookup_type = "host"
        if lookup_type == "ip_address":
            try:
                all_geo_info_fields = self.gi.record_by_addr(hostname_or_ip)
            except Exception,e:
                pass
        else:
            try:
                all_geo_info_fields = self.gi.record_by_name(hostname_or_ip)
            except:
                pass
        if not self.geo_info_fields:
            return all_geo_info_fields
        configured_geo_info_fields = {}
        for field_name in self.geo_info_fields:
            try:
                configured_geo_info_fields[field_name] = all_geo_info_fields[field_name]
            except:
                pass
        return configured_geo_info_fields


    def is_valid_ipv4_address(self, address):
        try:
            addr = socket.inet_pton(socket.AF_INET, address)
        except AttributeError: 
            try:
                addr= socket.inet_aton(address)
            except socket.error:
                return False
            return address.count('.') == 3
        except socket.error: 
            return False
        return True

    def is_valid_ipv6_address(self, address):
        try:
            addr= socket.inet_pton(socket.AF_INET6, address)
        except socket.error: 
            return False
        return True


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  tcp_socket_throughput.py
#  TCP Socket Connection Throughput Tester
#  Corey Goldberg (www.goldb.org), 2008

import sys
import time
import socket
from ctypes import c_int, c_bool
from multiprocessing import Process, Value, Lock
import logging
import logging.handlers
import os
import random

#host = sys.argv[1]
#port = int(sys.argv[2]) if len(sys.argv) == 3 else 6379

process_count = 5  # concurrent sender agents

counter_lock = Lock()
def increment(counter):
    with counter_lock:
        counter.value += 1

def reset(counter):
    with counter_lock:
        counter.value = 0

class Controller:
    def __init__(self):
        self.count_ref = Value(c_int)

    def start(self):
        for i in range(process_count):
            agent = Agent(self.count_ref)
            agent.start()
        print 'started %d threads' % (i + 1)
        while self.count_ref.value > 0:
            line = 'connects/sec: %s' % self.count_ref.value
            reset(self.count_ref)
            print chr(0x08) * len(line)
            print line
            time.sleep(1)

class Agent(Process):
    def __init__(self, count_ref):
        Process.__init__(self)
        self.daemon = True
        self.logger = logging.getLogger('MyLogger')
        self.logger.setLevel(logging.DEBUG)
        handler = logging.handlers.SysLogHandler(address = '/dev/log')
        self.logger.addHandler(handler)
        self.count_ref = count_ref
        random.seed(os.getpid())

    def run(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        for counter in xrange(0, 1000):
            try:
                rand = "%f-%04.3f" % (time.time(), random.random())
                message ='192.168.2.20 - - [28/Jul/2006:10:27:10 -0300] "GET /cgi-bin/try/%s HTTP/1.0" 200 3395' % rand
                self.logger.error(message)
                increment(self.count_ref)
                s.close()
                time.sleep(.01) #.0000001
            except:
                etype, evalue, etb = sys.exc_info()
                print 'Exception: %s, Error: %s.' % (etype, evalue)

if __name__ == '__main__':
    controller = Controller()
    controller.start()




# -*- coding: utf-8 -*-
import logging
import sys
import time
import elasticsearch

from lumbermill.constants import IS_PYPY
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Buffers import Buffer
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue, mapDynamicValueInString

# For pypy the default json module is the fastest.
if IS_PYPY:
    import json
else:
    json = False
    for module_name in ['ujson', 'yajl', 'simplejson', 'json']:
        try:
            json = __import__(module_name)
            break
        except ImportError:
            pass
    if not json:
        raise ImportError


@ModuleDocstringParser
class ElasticSearchSink(BaseThreadedModule):
    """
    Store the data dictionary in an elasticsearch index.

    The elasticsearch module takes care of discovering all nodes of the elasticsearch cluster.
    Requests will the be loadbalanced via round robin.

    action:     Either index or update. If update be sure to provide the correct doc_id.
    format:     Which event fields to send on, e.g. '$(@timestamp) - $(url) - $(country_code)'.
                If not set the whole event dict is send.
    nodes:      Configures the elasticsearch nodes.
    connection_type:    One of: 'thrift', 'http'.
    http_auth:  'user:password'.
    use_ssl:    One of: True, False.
    index_name: Sets the index name. Timepatterns like %Y.%m.%d and dynamic values like $(bar) are allowed here.
    doc_id:     Sets the es document id for the committed event data.
    routing:    Sets a routing value (@see: http://www.elasticsearch.org/blog/customizing-your-document-routing/)
                Timepatterns like %Y.%m.%d are allowed here.
    ttl:        When set, documents will be automatically deleted after ttl expired.
                Can either set time in milliseconds or elasticsearch date format, e.g.: 1d, 15m etc.
                This feature needs to be enabled for the index.
                @See: http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/mapping-ttl-field.html
    sniff_on_start: The client can be configured to inspect the cluster state to get a list of nodes upon startup.
                    Might cause problems on hosts with multiple interfaces. If connections fail, try to deactivate this.
    sniff_on_connection_fail: The client can be configured to inspect the cluster state to get a list of nodes upon failure.
                              Might cause problems on hosts with multiple interfaces. If connections fail, try to deactivate this.
    consistency:    One of: 'one', 'quorum', 'all'.
    store_interval_in_secs:     Send data to es in x seconds intervals.
    batch_size: Sending data to es if event count is above, even if store_interval_in_secs is not reached.
    backlog_size:   Maximum count of events waiting for transmission. If backlog size is exceeded no new events will be processed.

    Configuration template:

    - ElasticSearchSink:
       action:                          # <default: 'index'; type: string; is: optional; values: ['index', 'update']>
       format:                          # <default: None; type: None||string; is: optional>
       nodes:                           # <type: string||list; is: required>
       connection_type:                 # <default: 'urllib3'; type: string; values: ['urllib3', 'requests']; is: optional>
       http_auth:                       # <default: None; type: None||string; is: optional>
       use_ssl:                         # <default: False; type: boolean; is: optional>
       index_name:                      # <default: 'lumbermill-%Y.%m.%d'; type: string; is: optional>
       doc_id:                          # <default: '$(lumbermill.event_id)'; type: string; is: optional>
       routing:                         # <default: None; type: None||string; is: optional>
       ttl:                             # <default: None; type: None||integer||string; is: optional>
       sniff_on_start:                  # <default: False; type: boolean; is: optional>
       sniff_on_connection_fail:        # <default: False; type: boolean; is: optional>
       consistency:                     # <default: 'quorum'; type: string; values: ['one', 'quorum', 'all']; is: optional>
       store_interval_in_secs:          # <default: 5; type: integer; is: optional>
       batch_size:                      # <default: 500; type: integer; is: optional>
       backlog_size:                    # <default: 500; type: integer; is: optional>
    """

    module_type = "output"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method.
        BaseThreadedModule.configure(self, configuration)
        for module_name in ['elasticsearch', 'urllib3', 'requests']:
            if self.getConfigurationValue('log_level') == 'info':
                logging.getLogger(module_name).setLevel(logging.WARN)
            else:
                # Set log level for elasticsarch library if configured to other than default.
                logging.getLogger(module_name).setLevel(self.logger.level)
        self.action = self.getConfigurationValue('action')
        self.format = self.getConfigurationValue('format')
        self.consistency = self.getConfigurationValue("consistency")
        self.ttl = self.getConfigurationValue("ttl")
        self.index_name = self.getConfigurationValue("index_name")
        self.routing_pattern = self.getConfigurationValue("routing")
        self.doc_id_pattern = self.getConfigurationValue("doc_id")
        self.es_nodes = self.getConfigurationValue("nodes")
        if not isinstance(self.es_nodes, list):
            self.es_nodes = [self.es_nodes]
        if self.getConfigurationValue("connection_type") == 'urllib3':
            self.connection_class = elasticsearch.connection.Urllib3HttpConnection
        elif self.getConfigurationValue("connection_type") == 'requests':
            self.connection_class = elasticsearch.connection.RequestsHttpConnection

    def getStartMessage(self):
        return "Idx: %s. Max buffer size: %d" % (self.index_name, self.getConfigurationValue('backlog_size'))

    def initAfterFork(self):
        BaseThreadedModule.initAfterFork(self)
        # Init es client after fork as mentioned in https://elasticsearch-py.readthedocs.org/en/master/
        self.es = self.connect()
        if not self.es:
            self.lumbermill.shutDown()
            return
        # As the buffer uses a threaded timed function to flush its buffer and thread will not survive a fork, init buffer here.
        self.buffer = Buffer(self.getConfigurationValue('batch_size'), self.storeData, self.getConfigurationValue('store_interval_in_secs'), maxsize=self.getConfigurationValue('backlog_size'))

    def connect(self):
        es = False
        tries = 0
        while tries < 5 and not es:
            try:
                # Connect to es node and round-robin between them.
                self.logger.debug("Connecting to %s." % self.es_nodes)
                es = elasticsearch.Elasticsearch(self.es_nodes,
                                                 connection_class=self.connection_class,
                                                 sniff_on_start=self.getConfigurationValue('sniff_on_start'),
                                                 sniff_on_connection_fail=self.getConfigurationValue('sniff_on_connection_fail'),
                                                 sniff_timeout=5,
                                                 maxsize=20,
                                                 use_ssl=self.getConfigurationValue('use_ssl'),
                                                 http_auth=self.getConfigurationValue('http_auth'))
            except:
                etype, evalue, etb = sys.exc_info()
                self.logger.warning("Connection to %s failed. Exception: %s, Error: %s." % (self.es_nodes, etype, evalue))
                self.logger.warning("Waiting %s seconds before retring to connect." % ((4 + tries)))
                time.sleep(4 + tries)
                tries += 1
                continue
        if not es:
            self.logger.error("Connection to %s failed. Shutting down." % self.es_nodes)
            self.lumbermill.shutDown()
        else:
            self.logger.debug("Connection to %s successful." % self.es_nodes)
        return es

    def handleEvent(self, event):
        if self.format:
            publish_data = self.getConfigurationValue('format', event)
        else:
            publish_data = event
        self.buffer.append(publish_data)
        yield None

    def dataToElasticSearchJson(self, events):
        """
        Format data for elasticsearch bulk update.
        """
        json_data = []
        for event in events:
            index_name = mapDynamicValueInString(self.index_name, event, use_strftime=True).lower()
            event_type = event['lumbermill']['event_type'] if 'event_type' in event['lumbermill'] else 'Unknown'
            doc_id = mapDynamicValue(self.doc_id_pattern, event)
            routing = mapDynamicValue(self.routing_pattern, use_strftime=True)
            if not doc_id:
                self.logger.error("Could not find doc_id %s for event %s." % (self.getConfigurationValue("doc_id"), event))
                continue
            header = {self.action: {'_index': index_name,
                                    '_type': event_type,
                                    '_id': doc_id}}
            if self.routing_pattern:
                header['index']['_routing'] = routing
            if self.ttl:
                header['index']['_ttl'] = self.ttl
            if self.action == 'update':
                event = {'doc': event}
            try:
                json_data.append("\n".join((json.dumps(header), json.dumps(event), "\n")))
            except UnicodeDecodeError:
                etype, evalue, etb = sys.exc_info()
                self.logger.error("Could not json encode %s. Exception: %s, Error: %s." % (event, etype, evalue))
        json_data = "".join(json_data)
        return json_data

    def storeData(self, events):
        json_data = self.dataToElasticSearchJson(events)
        try:
            #started = time.time()
            # Bulk update of 500 events took 0.139621019363.
            self.es.bulk(body=json_data, consistency=self.consistency)
            #print("Bulk update of %s events took %s." % (len(events), time.time() - started))
            return True
        except elasticsearch.exceptions.ConnectionError:
            try:
                self.logger.warning("Lost connection to %s. Trying to reconnect." % (self.es_nodes, index_name))
                self.es = self.connect()
            except:
                time.sleep(.5)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Server communication error. Exception: %s, Error: %s." % (etype, evalue))
            self.logger.debug("Payload: %s" % json_data)
            if "Broken pipe" in evalue or "Connection reset by peer" in evalue:
                self.es = self.connect()

    def shutDown(self):
        try:
            self.buffer.flush()
        except:
            pass
        BaseThreadedModule.shutDown(self)


# -*- coding: utf-8 -*-
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class SyslogPrivalParser(BaseThreadedModule):
    """
    It will parse the source field in the event dictionary for the default severity
    and facility fields (RFC5424, http://tools.ietf.org/html/rfc5424).
    The source field must contain the prival with the pattern: "\d+"

    Numerical             Facility
     Code

      0             kernel messages
      1             user-level messages
      2             mail system
      3             system daemons
      4             security/authorization messages
      5             messages generated internally by syslogd
      6             line printer subsystem
      7             network news subsystem
      8             UUCP subsystem
      9             clock daemon
     10             security/authorization messages
     11             FTP daemon
     12             NTP subsystem
     13             log audit
     14             log alert
     15             clock daemon (note 2)
     16             local use 0  (local0)
     17             local use 1  (local1)
     18             local use 2  (local2)
     19             local use 3  (local3)
     20             local use 4  (local4)
     21             local use 5  (local5)
     22             local use 6  (local6)
     23             local use 7  (local7)

    Numerical         Severity
     Code

      0       Emergency: system is unusable
      1       Alert: action must be taken immediately
      2       Critical: critical conditions
      3       Error: error conditions
      4       Warning: warning conditions
      5       Notice: normal but significant condition
      6       Informational: informational messages
      7       Debug: debug-level messages

    Configuration template:

    - SyslogPrivalParser:
       source_field:                    # <default: 'syslog_prival'; type: string; is: optional>
       map_values: False                # <default: True; type: boolean; is: optional>
       facility_mappings:               # <default: {}; type: dictionary; is: optional>
       severity_mappings:               # <default: {}; type: dictionary; is: optional>
       receivers:
        - NextModule
    """

    module_type = "parser"
    """Set module type"""

    rfc_5424_facilities = { 0:  "kernel messages",
                          1: "user-level",
                          2: "mail",
                          3: "system",
                          4: "security/authorization",
                          5: "syslogd",
                          6: "line printer",
                          7: "network news",
                          8: "UUCP",
                          9: "clock",
                         10: "security/authorization",
                         11: "FTP",
                         12: "NTP",
                         13: "log audit",
                         14: "log alert",
                         15: "clock",
                         16: "local0",
                         17: "local1",
                         18: "local2",
                         19: "local3",
                         20: "local4",
                         21: "local5",
                         22: "local6",
                         23: "local7"}

    rfc_5424_severities = { 0: "Emergency",
                          1: "Alert",
                          2: "Critical",
                          3: "Error",
                          4: "Warning",
                          5: "Notice",
                          6: "Informational",
                          7: "Debug"}

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.source_field = self.getConfigurationValue('source_field')
        self.facility_mappings = dict(self.rfc_5424_facilities.items() + self.getConfigurationValue('facility_mappings').items())
        self.severity_mappings = dict(self.rfc_5424_severities.items() + self.getConfigurationValue('severity_mappings').items())

    def handleEvent(self, event):
        try:
            prival = int(event[self.source_field])
        except:
            yield event
            return
        # Calculate facility and priority from PRIVAL (@see: http://tools.ietf.org/html/rfc5424#section-6.2.1)
        event['syslog_facility'] = prival >> 3
        event['syslog_severity'] = prival & 7
        if not self.getConfigurationValue('map_values'):
            yield event
            return
        try:
            event['syslog_facility'] = self.facility_mappings[event['syslog_facility']]
            event['syslog_severity'] = self.severity_mappings[event['syslog_severity']]
        except KeyError:
            pass
        yield event



# -*- coding: utf-8 -*-
import codecs
import sys
import functools
import threading
import re
import ast
import multiprocessing
import collections


def Singleton(class_):
    instances = {}

    def getinstance(*args, **kwargs):
        if class_ not in instances:
            instances[class_] = class_(*args, **kwargs)
        return instances[class_]

    return getinstance

def setInterval(interval, max_run_count=0, call_on_init=False):
    def decorator(function):
        def wrapper(*args, **kwargs):
            stopped = threading.Event()
            def loop(): # executed in another thread
                run_count = 0
                if call_on_init:
                    function(*args, **kwargs)
                while not stopped.wait(interval): # until stopped
                    function(*args, **kwargs)
                    if max_run_count > 0:
                        run_count += 1
                        if run_count == max_run_count:
                            break
            t = threading.Thread(target=loop)
            t.daemon = True # stop if the program exits
            t.start()
            return stopped
        return wrapper
    return decorator

def setProcessInterval(interval):
    def decorator(function):
        def wrapper(*args, **kwargs):
            stopped = multiprocessing.Event()
            def loop(): # executed in another thread
                while not stopped.wait(interval): # until stopped
                    function(*args, **kwargs)
            t = multiprocessing.Process(target=loop)
            t.daemon = True # stop if the program exits
            t.start()
            return stopped
        return wrapper
    return decorator


def ModuleDocstringParser(cls):
    @functools.wraps(cls)
    def wrapper(*args, **kwargs):
        instance = cls(*args, **kwargs)
        instance.configuration_metadata = {}
        config_option_regex = "\s*(?P<name>.*?):.*?#\s*<(?P<props>.*?)>"
        regex = re.compile(config_option_regex, re.MULTILINE)
        docstring = ""
        # Get docstring from parents. Only single inheritance supported.
        for parent_class in cls.__bases__:
            if parent_class.__doc__:
                docstring += parent_class.__doc__
        if cls.__doc__:
            docstring += cls.__doc__
        for matches in regex.finditer(docstring):
            config_option_info = matches.groupdict()
            for prop_info in config_option_info['props'].split(";"):
                try:
                    prop_name, prop_value = [m.strip() for m in prop_info.split(":", 1)]
                    # Replace escaped backslashes.
                    if "//" in prop_value:
                        prop_value = codecs.escape_decode(prop_value)
                except ValueError:
                    instance.logger.debug("Could not parse config setting %s." % config_option_info)
                    continue
                if prop_name in ["default", "values"]:
                    try:
                        prop_value = ast.literal_eval(prop_value)
                    except:
                        etype, evalue, etb = sys.exc_info()
                        instance.logger.error("Could not parse %s from docstring. Exception: %s, Error: %s." % (prop_value, etype, evalue))
                        continue
                    # Set default values in module configuration. Will be overwritten by custom values
                    if prop_name == "default":
                        instance.configuration_data[config_option_info['name'].strip()] = prop_value
                    # Support for multiple datatypes using the pattern: "type: string||list;"
                if prop_name == "type":
                    prop_value = prop_value.split("||")
                    prop_value.append('Unicode')
                try:
                    instance.configuration_metadata[config_option_info['name'].strip()].update({prop_name: prop_value})
                except:
                    instance.configuration_metadata[config_option_info['name'].strip()] = {prop_name: prop_value}
        # Set self_hostname
        return instance

    return wrapper

def memoize(obj):
    cache = obj.cache = {}
    @functools.wraps(obj)
    def memoizer(*args, **kwargs):
        key = str(args) + str(kwargs)
        if key not in cache:
            cache[key] = obj(*args, **kwargs)
        return cache[key]
    return memoizer

class BoundedOrderedDict(collections.OrderedDict):
    def __init__(self, *args, **kwds):
        self.maxlen = kwds.pop("maxlen", None)
        collections.OrderedDict.__init__(self, *args, **kwds)
        self._checklen()

    def __setitem__(self, key, value):
        collections.OrderedDict.__setitem__(self, key, value)
        self._checklen()

    def _checklen(self):
        if self.maxlen is not None:
            while len(self) > self.maxlen:
                self.popitem(last=False)

def memoize(func=None, maxlen=None):
    """ Bounded memoize decorator.
        Found @http://stackoverflow.com/questions/9389307/how-do-i-create-a-bounded-memoization-decorator-in-python
    """
    if func:
        cache = BoundedOrderedDict(maxlen=maxlen)
        @functools.wraps(func)
        def memo_target(*args):
            lookup_value = args
            if lookup_value not in cache:
                cache[lookup_value] = func(*args)
            return cache[lookup_value]
        return memo_target
    else:
        def memoize_factory(func):
            return memoize(func, maxlen=maxlen)
        return memoize_factory


#!/usr/bin/env python

from __future__ import print_function
import sys
import time
import threading
import Queue
import redis

def usage():
    sys.stdout = sys.stderr
    print('Usage: spam_redis.py [channel||list] name count host [port 5151]')
    sys.exit(2)

if len(sys.argv) < 5:
    usage()
    sys.exit()

redis_type = sys.argv[1]
redis_name = sys.argv[2]
count = int(eval(sys.argv[3]))
host = sys.argv[4]
port = int(sys.argv[5]) if len(sys.argv) == 6 else 6379

class Worker(threading.Thread):

    def __init__(self, queue):
        threading.Thread.__init__(self)
        self.daemon = True
        self.queue = queue
        self.connected = False

    def run(self):
        self.connected = False
        try:
            self.client = redis.Redis(host=host, port=port)
            self.connected = True
        except:
            etype, evalue, etb = sys.exc_info()
            print('Could no connect to redis server at %s:%s. Exception: %s, Error: %s.' % (host, port, etype, evalue))
        while self.connected:
            try:
                message = self.queue.get(timeout=.5)
                if redis_type == 'channel':
                    self.client.publish(redis_name, message)
                else:
                    self.client.rpush(redis_name, message)
            except Queue.Empty:
                break

class RedisLoadTester():

    def __init__(self, num_workers=10):
        self.lock = threading.Lock()
        self.counter = 0
        self.queue = Queue.Queue(10)
        self.num_workers = num_workers

    def start(self, message):
        total_item_count = count
        workers = set()
        for i in xrange(0, self.num_workers):
            worker = Worker(self.queue)
            worker.start()
            workers.add(worker)
            time.sleep(.1)
            if not worker.connected:
                sys.exit()
        print("Start load test.")
        start = time.time()
        for counter in xrange(0, total_item_count):
            if counter % 1000 == 0:
                sys.stdout.write("Message already sent: %s. Mean req/s: %s\r" % (counter+1, counter / (time.time()-start)))
                sys.stdout.flush()
            now = "%f" % time.time()
            self.queue.put(message % now)
        for worker in workers:
            worker.join()
        stop = time.time()
        print("Message sent: %s. Took %s. Mean req/s: %s" % (counter+1, stop-start, total_item_count / (stop-start)))

if __name__ == '__main__':
    lt = RedisLoadTester()
    lt.start("<13>229.25.18.182 - - [28/Jul/2006:10:27:10 -0300] \"GET /cgi-bin/try/9153/?param1=Test&param2=%s HTTP/1.0\" 200 3395 \"Mozilla/5.0 (Windows NT 6.1; WOW64; rv:32.0) Gecko/20100101 Firefox/32.0\"\n")





# -*- coding: utf-8 -*-
import pprint
import time

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue


@ModuleDocstringParser
class StdOutSink(BaseThreadedModule):
    """
    Print the data dictionary to stdout.

    pretty_print: Use pythons pprint function.
    format: Format of messages to send to graphite, e.g.: ['lumbermill.stats.event_rate_$(interval)s $(event_rate)'].

    Configuration template:

    - StdOutSink:
       pretty_print:                    # <default: True; type: boolean; is: optional>
       format:                          # <default: None; type: None||string; is: optional>
       parser:                          # <default: None; type: None||string; is: optional>
    """

    module_type = "output"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.format = self.getConfigurationValue('format')
        self.pretty_print = self.getConfigurationValue('pretty_print')
        self.printing = False

    def handleEvent(self, event):
        while self.printing:
            time.sleep(.0001)
        self.printing = True
        if self.format:
            output = mapDynamicValue(self.format, event)
        else:
            output = event
        if self.pretty_print and not self.format:
            pprint.pprint(output, indent=4)
        else:
            print("%s" % output)
        self.printing = False
        yield None



# -*- coding: utf-8 -*-
import sys

import nmap

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseModule import BaseModule
from lumbermill.utils.Decorators import ModuleDocstringParser, setInterval
from lumbermill.utils.misc import TimedFunctionManager


@ModuleDocstringParser
class NmapScanner(BaseModule):
    """
    Scan network with nmap and emit result as new event.

    Configuration template:

    - NmapScanner:
       network:                         # <type: string; is: required>
       netmask:                         # <default: '/24'; type: string; is: optional>
       ports:                           # <default: None; type: None||string; is: optional>
       arguments:                       # <default: '-O -F --osscan-limit'; type: string; is: optional>
       interval:                        # <default: 900; type: integer; is: optional>
       receivers:
        - NextModule
    """

    module_type = "input"
    """Set module type"""
    # TODO: This module can run in forked processes. We need some way to partition the network and give each process a
    # segment to scan.
    can_run_forked = False

    def configure(self, configuration):
        # Call parent configure method
        BaseModule.configure(self, configuration)
        self.network = self.getConfigurationValue('network')
        self.netmask = self.getConfigurationValue('netmask')
        self.arguments = self.getConfigurationValue('arguments')

    def getScannerFunc(self):
        @setInterval(self.getConfigurationValue('interval'), call_on_init=True)
        def scanNetwork():
            # Get all alive hosts
            try:
                scan_results = self.scanner.scan('%s%s' % (self.network,self.netmask), arguments="-sn")
            except nmap.PortScannerError:
                etype, evalue, etb = sys.exc_info()
                self.logger.warning("Scanning failed. Exception: %s, Error: %s." % (etype, evalue))
                return
            for host, scan_result in scan_results['scan'].items():
                try:
                    host_scan_result = self.scanner.scan('%s/32' % (host), arguments=self.arguments)
                except nmap.PortScannerError:
                    etype, evalue, etb = sys.exc_info()
                    self.logger.warning("Scanning failed. Exception: %s, Error: %s." % (etype, evalue))
                    return
                if host in host_scan_result['scan']:
                    self.handleEvent(host, host_scan_result['scan'][host])
        return scanNetwork

    def handleEvent(self, host, scan_result):
        # Get OS from scan.
        if 'osmatch' in scan_result:
            os_info = sorted(scan_result['osmatch'], key=lambda k: int(k['accuracy']))
            scan_result['detected_os'] = os_info[0]['name']
            scan_result.pop('osmatch')
        if 'vendor' in scan_result and isinstance(scan_result['vendor'], dict) and len(scan_result['vendor']) > 0:
            scan_result['vendor'] = scan_result['vendor'].values()[0]
        # Drop some fields.
        if 'osclass' in scan_result:
            scan_result.pop('osclass')
        event = DictUtils.getDefaultEventDict(scan_result, caller_class_name=self.__class__.__name__)
        event['lumbermill']['event_type'] = 'nmap_scan'
        self.sendEvent(event)

    def start(self):
        self.scanner = nmap.PortScanner()
        timed_func = self.getScannerFunc()
        self.timed_func_handler = TimedFunctionManager.startTimedFunction(timed_func)


# -*- coding: utf-8 -*-
import socket
import sys
import urllib2

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class HttpRequest(BaseThreadedModule):
    """
    Issue an arbitrary http request and store the response in a configured field.

    This module supports the storage of the responses in an redis db. If redis_store is set,
    it will first try to retrieve the response from redis via the key setting.
    If that fails, it will execute the http request and store the result in redis.

    Configuration template:

    - HttpRequest:
       url:                             # <type: string; is: required>
       socket_timeout:                  # <default: 25; type: integer; is: optional>
       target_field:                    # <default: "gambolputty_http_request"; type: string; is: optional>
       redis_store:                     # <default: None; type: None||string; is: optional>
       redis_key:                       # <default: None; type: None||string; is: optional if redis_store is None else required>
       redis_ttl:                       # <default: 60; type: integer; is: optional>
       receivers:
        - NextModule
    """
    module_type = "modifier"
    """Set module type"""

    def configure(self, configuration):
        BaseThreadedModule.configure(self, configuration)
        socket.setdefaulttimeout(self.getConfigurationValue('socket_timeout'))
        # Get redis client module.
        if self.getConfigurationValue('redis_store'):
            mod_info = self.lumbermill.getModuleInfoById(self.getConfigurationValue('redis_store'))
            self.redis_store = mod_info['instances'][0]
        else:
            self.redis_store = None

    def handleEvent(self, event):
        request_url = self.getConfigurationValue('url', event)
        target_field_name = self.getConfigurationValue('target_field', event)
        if not request_url or not target_field_name:
            yield event
            return
        result = None
        if self.redis_store:
            redis_key = self.getConfigurationValue('redis_key', event)
            result = self.redis_store.get(redis_key)
        if result is None:
            try:
                result = self.execRequest(request_url).read()
                if result and self.redis_store:
                    self.redis_store.set(redis_key, result, self.getConfigurationValue('redis_ttl'))
            except:
                yield event
                return
        event[target_field_name] =  result
        yield event

    def execRequest(self, url):
        try:
            response = urllib2.urlopen(url)
            return response
        except urllib2.HTTPError, e:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Request to %s failed. Exception: %s, Error: %s" % (url, etype, evalue))
            raise
        except urllib2.URLError, e:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Request to %s failed. Exception: %s, Error: %s" % (url, etype, evalue))
            raise
        return False


# -*- coding: utf-8 -*-
import sys

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser, setInterval
from lumbermill.utils.misc import TimedFunctionManager


@ModuleDocstringParser
class FacetV2(BaseThreadedModule):
    """
    Collect different values of one field over a defined period of time and pass all
    encountered variations on as new event after period is expired.

    The "add_event_fields" configuration will copy the configured event fields into the "other_event_fields" list.

    The event emitted by this module will be of type: "facet" and will have "facet_field",
    "facet_count", "facets" and "other_event_fields" fields set.

    This module supports the storage of the facet info in an backend db (At the moment this only works for a redis backend.
    This offers the possibility of using this module across multiple instances of LumberMill.

    source_field: Field to be scanned for unique values.
    group_by: Field to relate the variations to, e.g. ip address.
    add_event_fields: Fields to add from the original event to the facet event.
    interval: Number of seconds to until all encountered values of source_field will be send as new facet event.
    backend: Name of a key::value store plugin. When running multiple instances of gp this backend can be used to
             synchronize events across multiple instances.
    backend_ttl: Time to live for backend entries. Should be greater than interval.

    Configuration template:

    - Facet:
       source_field:                    # <type:string; is: required>
       group_by:                        # <type:string; is: required>
       add_event_fields:                # <default: []; type: list; is: optional>
       interval:                        # <default: 5; type: float||integer; is: optional>
       backend:                         # <default: None; type: None||string; is: optional>
       backend_ttl:                     # <default: 60; type: integer; is: optional>
       receivers:
        - NextModule
    """
    facet_data = {}
    """Holds the facet data for all instances"""

    backend_keys = []
    """Holds just the backend keys to all facet data"""

    module_type = "misc"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.source_field = self.getConfigurationValue('source_field')
        self.group_by_field = self.getConfigurationValue('group_by')
        self.add_event_fields = self.getConfigurationValue('add_event_fields')
        self.backend_ttl = self.getConfigurationValue('backend_ttl')
        self.persistence_backend = None
        if self.getConfigurationValue('backend'):
            backend_info = self.lumbermill.getModuleInfoById(self.getConfigurationValue('backend'))
            if not backend_info:
                self.logger.error("Could not find %s backend for persistant storage." % (self.getConfigurationValue('backend')))
                self.lumbermill.shutDown()
                return
            self.persistence_backend = backend_info['instances'][0]

    def getFacetDataFromRedis(self, key):
        facet_info = self.persistence_backend.get(key)
        if not facet_info:
            facet_info = False
        return facet_info

    def getFacetDataInternal(self, key):
        try:
            facet_info = FacetV2.facet_data[key]
        except KeyError:
            facet_info = False
        return facet_info

    def getFacetData(self, key):
        facet_data = self.getFacetDataInternal(key)
        if not facet_data and self.persistence_backend:
            facet_data = self.getFacetDataFromRedis(key)
        if not facet_data:
            facet_data = {'other_event_fields': {}, 'facets': []}
        return facet_data

    def setFacetDataInRedis(self, key, facet_data):
        try:
            self.persistence_backend.set(key, facet_data, self.backend_ttl)
            if key not in FacetV2.backend_keys:
                FacetV2.backend_keys.append(key)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.warning("Could not store facet data in persistance backend. Exception: %s, Error: %s." % (etype, evalue))
            pass

    def setFacetData(self, key, facet_info):
        FacetV2.facet_data[key] = facet_info

    def sendFacetEventToReceivers(self, facet_data):
        event = DictUtils.getDefaultEventDict({'facet_field': self.source_field,
                                          'facet_count': len(facet_data['facets']),
                                          'facets': facet_data['facets']},
                                          caller_class_name=self.__class__.__name__,
                                          event_type='facet')
        if facet_data['other_event_fields']:
            event['other_event_fields'] = facet_data['other_event_fields']
        self.sendEvent(event)

    def evaluateFacets(self):
        """
        This is a method on its on and not implemented in timedEvaluateFacets because the shutDown method needs to call
        this as well and it needs to be executed directly.
        """
        if self.persistence_backend:
            backend_lock = self.persistence_backend.getLock("FacetLock", timeout=10)
            if not backend_lock:
                return
            lock_acquired = backend_lock.acquire(blocking=True)
            if not lock_acquired:
                return
            for key in FacetV2.backend_keys:
                facet_data = self.getFacetDataFromRedis(key)
                if facet_data:
                    self.sendFacetEventToReceivers(facet_data)
                # Clear backend items
                self.persistence_backend.client.delete(key)
            FacetV2.backend_keys = []
            backend_lock.release()
            return
        # Just internal facet data.
        for key, facet_data in FacetV2.facet_data.items():
            self.sendFacetEventToReceivers(facet_data)
        FacetV2.facet_data = {}

    def getEvaluateFunc(self):
        @setInterval(self.getConfigurationValue('interval'))
        def timedEvaluateFacets():
            self.evaluateFacets()
        return timedEvaluateFacets

    def getStoreFunc(self):
        @setInterval(1)
        def storeFacetsInRedis():
            backend_lock = self.persistence_backend.getLock("FacetLock", timeout=10)
            if not backend_lock:
                return
            lock_acquired = backend_lock.acquire(blocking=True)
            if not lock_acquired:
                return
            for key, facet_data in FacetV2.facet_data.items():
                current_facet_data = self.getFacetDataFromRedis(key)
                if not current_facet_data:
                    current_facet_data = {'other_event_fields': {}, 'facets': []}
                if 'other_event_fields' in current_facet_data:
                    current_facet_data['other_event_fields'].update(facet_data['other_event_fields'])
                current_facet_data['facets'] += facet_data['facets']
                self.setFacetDataInRedis(key, current_facet_data)
            FacetV2.facet_data = {}
            backend_lock.release()
        return storeFacetsInRedis


    def initAfterFork(self):
        self.evaluate_facet_data_func = self.getEvaluateFunc()
        self.timed_func_handler_a = TimedFunctionManager.startTimedFunction(self.evaluate_facet_data_func)
        if self.persistence_backend:
            self.store_facets_in_backend_func = self.getStoreFunc()
            self.timed_func_handler_b = TimedFunctionManager.startTimedFunction(self.store_facets_in_backend_func)
        BaseThreadedModule.initAfterFork(self)

    def handleEvent(self, event):
        """
        Process the event.

        @param event: dictionary
        @return data: dictionary
        """
        # Get key and value for facet from event.
        try:
            facet_value = event[self.source_field]
        except KeyError:
            yield event
            return
        key = event[self.group_by_field]
        if not key and self.persistence_backend:
            self.logger.warning("Group_by value %s could not be generated. Event ignored." % (self.getConfigurationValue('group_by')))
            yield event
            return
        key = "FacetValues:%s" % key
        facet_data = self.getFacetData(key)
        if facet_value not in facet_data['facets']:
            keep_fields = {}
            for field_name in self.add_event_fields:
                try:
                    keep_fields[field_name] = event[field_name]
                except KeyError:
                    pass
            if keep_fields:
                facet_data['other_event_fields'][facet_value] = keep_fields
            facet_data['facets'].append(facet_value)
            self.setFacetData(key, facet_data)
        yield event

    def shutDown(self):
        # Push any remaining facet data.
        self.evaluateFacets()
        # Call parent configure method.
        BaseThreadedModule.shutDown(self)


# -*- coding: utf-8 -*-
import urllib
import urlparse

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class UrlParser(BaseThreadedModule):
    """
    Urlencode or decode an event field and extract url parameters.

    action: Either encode or decode data.
    source_field: Event field to en/decode.
    target_field: Event field to update with en/decode result. If not set source will be replaced.
    parse_querystring: Parse url for query parameters and extract them.
    querystring_target_field: Event field to update with url parameters.
    querystring_prefix: Prefix string to prepend to url parameter keys.

    Configuration template:

    - UrlParser:
       action:                          # <default: 'decode'; type: string; values: ['decode','encode']; is: optional>
       source_field:                    # <type: string; is: required>
       target_field:                    # <default: None; type: None||string; is: optional>
       parse_querystring:               # <default: False; type: boolean; is: optional>
       querystring_target_field:        # <default: None; type: None||string; is: optional>
       querystring_prefix:              # <default: None; type: None||string; is: optional>
       receivers:
        - NextModule
    """

    module_type = "parser"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.source_field = self.getConfigurationValue('source_field')
        if not self.getConfigurationValue('target_field'):
            self.target_field = self.source_field
        else:
            self.target_field = self.getConfigurationValue('target_field')
        self.parse_querystring = self.getConfigurationValue('parse_querystring')
        self.querystring_target_field = self.getConfigurationValue('querystring_target_field')
        self.querystring_prefix = self.getConfigurationValue('querystring_prefix')
        if self.getConfigurationValue('action') == 'decode':
            self.handleEvent = self.decodeEvent
        else:
            self.handleEvent = self.encodeEvent

    def decodeEvent(self, event):
        if self.source_field in event:
            #try:
            #    decoded_field = urllib.unquote(event[self.source_field]).decode('utf8')
            #except UnicodeDecodeError:
            #    decoded_field = urllib.unquote(unicode(event[self.source_field]))
            decoded_field = urllib.unquote(unicode(event[self.source_field]))
            parsed_result = urlparse.urlparse('%s' % decoded_field)
            parsed_url = {'scheme': parsed_result.scheme, 'path': parsed_result.path, 'params': parsed_result.params, 'query': parsed_result.query}
            event[self.target_field] = parsed_url
            if self.parse_querystring:
                query_params_dict = urlparse.parse_qs(parsed_result.query)
                if self.querystring_prefix:
                    query_params_dict = dict(map(lambda (key, value): ("%s%s" % (self.querystring_prefix, str(key)), value), query_params_dict.items()))
                if self.querystring_target_field:
                    event[self.querystring_target_field] = query_params_dict
                else:
                    event.update(query_params_dict)
        yield event

    def encodeEvent(self, event):
        if self.source_field in event:
            urllib.quote(event[self.source_field]).encode('utf8')
        yield event


# -*- coding: utf-8 -*-
import sys
import time
import threading
import Queue
import boto3


def usage():
    sys.stdout = sys.stderr
    print('Usage: spam_sqs.py -c count region aws_id aws_key queue_name')
    sys.exit(2)

if len(sys.argv) < 7:
    usage()
    sys.exit()

aws_regions = ['us-east-1', 'us-west-1', 'us-west-2', 'eu-central-1', 'eu-west-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', 'sa-east-1', 'us-gov-west-1', 'cn-north-1']

count = int(eval(sys.argv[2]))
region = sys.argv[3]
aws_id = sys.argv[4]
aws_key = sys.argv[5]
queue_name = sys.argv[6]

if type(count) is not int or region not in aws_regions:
    usage()
    sys.exit(255)

class Worker(threading.Thread):

    def __init__(self, queue):
        threading.Thread.__init__(self)
        self.daemon = True
        self.queue = queue
        self.connected = False

    def run(self):
        try:
            sqs_client = boto3.resource('sqs', region_name=region,
                                               api_version=None,
                                               use_ssl=True,
                                               verify=None,
                                               endpoint_url=None,
                                               aws_access_key_id=aws_id,
                                               aws_secret_access_key=aws_key,
                                               aws_session_token=None,
                                               config=None)
            self.connected = True
        except:
            etype, evalue, etb = sys.exc_info()
            print("Could not connect to sqs service. Exception: %s, Error: %s." % (etype, evalue))
            return
        try:
            sqs_queue = sqs_client.get_queue_by_name(QueueName=queue_name)
        except:
            etype, evalue, etb = sys.exc_info()
            print("Could not connect to sqs queue %s. Exception: %s, Error: %s." % (queue_name, etype, evalue))
            return
        while self.connected:
            try:
                message = self.queue.get(timeout=.5)
                sqs_queue.send_message(MessageBody=message)
            except Queue.Empty:
                self.connected = False
                break
        return

class SqsLoadTester():

    def __init__(self, num_workers=10):
        self.lock = threading.Lock()
        self.counter = 0
        self.queue = Queue.Queue(10)
        self.num_workers = num_workers

    def start(self, message):
        total_item_count = count
        workers = set()
        for i in xrange(0, self.num_workers):
            worker = Worker(self.queue)
            worker.start()
            workers.add(worker)
            time.sleep(.1)
            if not worker.connected:
                sys.exit()
        print("Start load test.")
        start = time.time()
        for counter in xrange(0, total_item_count):
            if counter % 1000 == 0:
                sys.stdout.write("Message already sent: %s. Mean req/s: %s\r" % (counter+1, counter / (time.time()-start)))
                sys.stdout.flush()
            now = "%f" % time.time()
            self.queue.put(message % now)
        for worker in workers:
            worker.join()
        stop = time.time()
        print("Message sent: %s. Took %s. Mean req/s: %s" % (counter+1, stop-start, total_item_count / (stop-start)))

if __name__ == '__main__':
    lt = SqsLoadTester(5)
    lt.start("<13>229.25.18.182 - - [28/Jul/2006:10:27:10 -0300] \"GET /cgi-bin/try/9153/?param1=Test&param2=%s HTTP/1.0\" 200 3395 \"Mozilla/5.0 (Linux; U; Android 2.3.5; en-in; HTC_DesireS_S510e Build/GRJ90) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1\"")


import os

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser, setInterval
from lumbermill.utils.StatisticCollector import StatisticCollector, MultiProcessStatisticCollector
from lumbermill.utils.misc import AnsiColors, TimedFunctionManager


@ModuleDocstringParser
class SimpleStats(BaseThreadedModule):
    """
    Collect and log some simple lumbermill statistic data.

    Use this module if you just need some simple statistics on how many events are passing through lumbermill.
    Per default, statistics will just be send to stdout.

    Configuration template:

    - SimpleStats:
       interval:                        # <default: 10; type: integer; is: optional>
       event_type_statistics:           # <default: True; type: boolean; is: optional>
       receive_rate_statistics:         # <default: True; type: boolean; is: optional>
       waiting_event_statistics:        # <default: False; type: boolean; is: optional>
       emit_as_event:                   # <default: False; type: boolean; is: optional>
    """

    module_type = "misc"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.emit_as_event = self.getConfigurationValue('emit_as_event')
        self.interval = self.getConfigurationValue('interval')
        self.stats_collector = StatisticCollector()
        for counter_name in ['events_received', 'event_type_Unknown', 'event_type_httpd_access_log']:
            MultiProcessStatisticCollector().initCounter(counter_name)
        self.module_queues = {}

    def getRunTimedFunctionsFunc(self):
        @setInterval(self.interval)
        def runTimedFunctionsFunc():
            self.accumulateEventTypeStats()
            self.accumulateReceiveRateStats()
            if self.lumbermill.is_master:
                self.printIntervalStatistics()
        return runTimedFunctionsFunc

    def accumulateEventTypeStats(self):
        for event_type, count in self.stats_collector.getAllCounters().items():
            MultiProcessStatisticCollector().incrementCounter(event_type, count)
            self.stats_collector.resetCounter(event_type)

    def accumulateReceiveRateStats(self):
        MultiProcessStatisticCollector().incrementCounter('events_received', self.stats_collector.getCounter('events_received'))
        self.stats_collector.resetCounter('events_received')

    def printIntervalStatistics(self):
        self.logger.info("############# Statistics (PID: %s) #############" % os.getpid())
        if self.getConfigurationValue('receive_rate_statistics'):
            self.receiveRateStatistics()
        if self.getConfigurationValue('waiting_event_statistics'):
            self.eventsInQueuesStatistics()
        if self.getConfigurationValue('event_type_statistics'):
            self.eventTypeStatistics()

    def eventTypeStatistics(self):
        self.logger.info(">> EventTypes Statistics")
        for event_type, count in sorted(MultiProcessStatisticCollector().getAllCounters().items()):
            count = count.value
            if not event_type.startswith('event_type_'):
                continue
            event_name = event_type.replace('event_type_', '')
            self.logger.info("EventType: %s%s%s - Hits: %s%s%s" % (AnsiColors.YELLOW, event_name, AnsiColors.ENDC, AnsiColors.YELLOW, count, AnsiColors.ENDC))
            if self.emit_as_event:
                self.sendEvent(DictUtils.getDefaultEventDict({"total_count": count, "count_per_sec": (count/self.interval), "field_name": event_name, "interval": self.interval }, caller_class_name="Statistics", event_type="statistic"))
            MultiProcessStatisticCollector().resetCounter(event_type)

    def receiveRateStatistics(self):
        self.logger.info(">> Receive rate stats")
        events_received = MultiProcessStatisticCollector().getCounter('events_received')
        if not events_received:
            events_received = 0
        MultiProcessStatisticCollector().resetCounter('events_received')
        self.logger.info("Received events in %ss: %s%s (%s/eps)%s" % (self.getConfigurationValue('interval'), AnsiColors.YELLOW, events_received, (events_received/self.interval), AnsiColors.ENDC))
        if self.emit_as_event:
            self.sendEvent(DictUtils.getDefaultEventDict({"total_count": events_received, "count_per_sec": (events_received/self.interval), "field_name": "all_events", "interval": self.interval }, caller_class_name="Statistics", event_type="statistic"))

    def eventsInQueuesStatistics(self):
        if len(self.module_queues) == 0:
            return
        self.logger.info(">> Queue stats")
        for module_name, queue in sorted(self.module_queues.items()):
            self.logger.info("Events in %s queue: %s%s%s" % (module_name, AnsiColors.YELLOW, queue.qsize(), AnsiColors.ENDC))
            if self.emit_as_event:
                self.sendEvent(DictUtils.getDefaultEventDict({"queue_count": queue.qsize(),  "field_name": "queue_counts", "interval": self.interval }, caller_class_name="Statistics", event_type="statistic"))

    def initAfterFork(self):
        # Get all configured queues for waiting event stats.
        for module_name, module_info in self.lumbermill.modules.items():
            instance = module_info['instances'][0]
            if not hasattr(instance, 'getInputQueue') or not instance.getInputQueue():
                continue
            self.module_queues[module_name] = instance.getInputQueue()
        TimedFunctionManager.startTimedFunction(self.getRunTimedFunctionsFunc())
        BaseThreadedModule.initAfterFork()


    def handleEvent(self, event):
        self.stats_collector.incrementCounter('events_received')
        if self.getConfigurationValue('event_type_statistics'):
            try:
                self.stats_collector.incrementCounter('event_type_%s' % event['lumbermill']['event_type'])
            except:
                pass
        yield event


# -*- coding: utf-8 -*-
import logging
import sys
import time

import boto3

import lumbermill.utils.DictUtils as DictUtils
from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class SQS(BaseThreadedModule):
    """
    Read messages from amazon sqs service.

    aws_access_key_id: Your AWS id.
    aws_secret_access_key: Your AWS password.
    region: The region in which to find your sqs service.
    queue: Queue name.
    attribute_names: A list of attributes that need to be returned along with each message.
    message_attribute_names: A list of message attributes that need to be returned.
    poll_interval_in_secs: How often should the queue be checked for new messages.
    batch_size: Number of messages to retrieve in one call.

    Configuration template:

    - SQS:
       aws_access_key_id:               # <type: string; is: required>
       aws_secret_access_key:           # <type: string; is: required>
       region:                          # <type: string; is: required; values: ['us-east-1', 'us-west-1', 'us-west-2', 'eu-central-1', 'eu-west-1', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', 'sa-east-1', 'us-gov-west-1', 'cn-north-1']>
       queue:                           # <type: string; is: required>
       attribute_names:                 # <default: ['All']; type: list; is: optional>
       message_attribute_names:         # <default: ['All']; type: list; is: optional>
       poll_interval_in_secs:           # <default: 1; type: integer; is: optional>
       batch_size:                      # <default: 10; type: integer; is: optional>
       receivers:
        - NextModule
    """

    module_type = "input"
    """Set module type"""
    can_run_forked = True

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        # Set boto log level.
        logging.getLogger('boto3').setLevel(logging.CRITICAL)
        logging.getLogger('botocore').setLevel(logging.CRITICAL)
        self.sqs_queue_name = self.getConfigurationValue('queue')
        self.attribute_names = self.getConfigurationValue('attribute_names')
        self.message_attribute_names = self.getConfigurationValue('message_attribute_names')
        self.poll_interval = self.getConfigurationValue('poll_interval_in_secs')
        self.batch_size = self.getConfigurationValue('batch_size')
        try:
            self.sqs_client = boto3.client('sqs', region_name=self.getConfigurationValue('region'),
                                                  api_version=None,
                                                  use_ssl=True,
                                                  verify=None,
                                                  endpoint_url=None,
                                                  aws_access_key_id=self.getConfigurationValue('aws_access_key_id'),
                                                  aws_secret_access_key=self.getConfigurationValue('aws_secret_access_key'),
                                                  aws_session_token=None,
                                                  config=None)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not connect to sqs service. Exception: %s, Error: %s." % (etype, evalue))
            self.lumbermill.shutDown()
        try:
            self.sqs_queue_url = self.sqs_client.get_queue_url(QueueName=self.sqs_queue_name)['QueueUrl']
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not get queue url for sqs queue %s. Exception: %s, Error: %s." % (self.sqs_queue_name, etype, evalue))
            self.lumbermill.shutDown()

    def run(self):
        while self.alive:
            messages_to_delete = []
            response = self.sqs_client.receive_message(QueueUrl=self.sqs_queue_url,
                                                       MaxNumberOfMessages=self.batch_size,
                                                       AttributeNames=self.attribute_names,
                                                       MessageAttributeNames=self.message_attribute_names)
            if 'Messages' not in response:
                time.sleep(self.poll_interval)
                continue
            for message in response['Messages']:
                event = DictUtils.getDefaultEventDict({"data": message['Body']}, caller_class_name="Sqs")
                event['sqs'] = {'attributes': message['Attributes'],
                                'id': message['MessageId'],
                                'md5_of_body': message['MD5OfBody'],
                                'md5_of_message_attributes': message.get('MD5OfMessageAttributes', None),
                                'message_attributes': message.get('MessageAttributes', None)}
                messages_to_delete.append({'Id': message['MessageId'],
                                           'ReceiptHandle': message['ReceiptHandle']})
                self.sendEvent(event)
            self.sqs_client.delete_message_batch(QueueUrl=self.sqs_queue_url, Entries=messages_to_delete)
        self.lumbermill.shutDown()


# -*- coding: utf-8 -*-
import socket
import sys

import msgpack
import zmq

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Buffers import Buffer
from lumbermill.utils.Decorators import ModuleDocstringParser
from lumbermill.utils.DynamicValues import mapDynamicValue


@ModuleDocstringParser
class ZmqSink(BaseThreadedModule):
    """
    Sends events to zeromq.

    server: Server to connect to. Pattern: hostname:port.
    pattern: Either push or pub.
    mode: Whether to run a server or client. If running as server, pool size is restricted to a single process.
    topic: The channels topic.
    hwm: Highwatermark for sending socket.
    format: Which event fields to send on, e.g. '$(@timestamp) - $(url) - $(country_code)'. If not set the whole event dict is send msgpacked.
    store_interval_in_secs: Send data to redis in x seconds intervals.
    batch_size: Send data to redis if event count is above, even if store_interval_in_secs is not reached.
    backlog_size: Maximum count of events waiting for transmission. Events above count will be dropped.

    Configuration template:

    - ZmqSink:
       server:                          # <default: 'localhost:5570'; type: string; is: optional>
       pattern:                         # <default: 'push'; type: string; values: ['push', 'pub']; is: optional>
       mode:                            # <default: 'connect'; type: string; values: ['connect', 'bind']; is: optional>
       topic:                           # <default: None; type: None||string; is: optional>
       hwm:                             # <default: None; type: None||integer; is: optional>
       format:                          # <default: None; type: None||string; is: optional>
       store_interval_in_secs:          # <default: 5; type: integer; is: optional>
       batch_size:                      # <default: 500; type: integer; is: optional>
       backlog_size:                    # <default: 500; type: integer; is: optional>
    """

    module_type = "input"
    """Set module type"""
    can_run_forked = True

    def configure(self, configuration):
         # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.server = None
        self.topic = self.getConfigurationValue('topic')
        self.format = self.getConfigurationValue('format')
        self.mode = self.getConfigurationValue('mode')
        if self.mode == "bind":
            self.can_run_forked = False

    def initZmqContext(self):
        self.zmq_context = zmq.Context()
        if self.getConfigurationValue('pattern') == 'push':
            self.client = self.zmq_context.socket(zmq.PUSH)
        else:
            self.client = self.zmq_context.socket(zmq.PUB)
        if self.getConfigurationValue('hwm'):
            try:
                self.client.setsockopt(zmq.SNDHWM, self.getConfigurationValue('hwm'))
            except:
                self.client.setsockopt(zmq.HWM, self.getConfigurationValue('hwm'))
        server_name, server_port = self.getConfigurationValue('server').split(":")
        try:
            server_addr = socket.gethostbyname(server_name)
        except socket.gaierror:
            server_addr = server_name
        try:
            if self.getConfigurationValue('mode') == 'connect':
                self.client.connect('tcp://%s:%s' % (server_addr, server_port))
            else:
                self.client.bind('tcp://%s:%s' % (server_addr, server_port))
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not connect to zeromq at %s. Exception: %s, Error: %s." % (self.getConfigurationValue('server'), etype, evalue))
            self.lumbermill.shutDown()

    def getStartMessage(self):
        return "%s. Max buffer size: %d" % (self.server, self.getConfigurationValue('backlog_size'))

    def initAfterFork(self):
        BaseThreadedModule.initAfterFork(self)
        self.initZmqContext()
        self.buffer = Buffer(self.getConfigurationValue('batch_size'), self.storeData, self.getConfigurationValue('store_interval_in_secs'), maxsize=self.getConfigurationValue('backlog_size'))

    def storeData(self, buffered_data):
        try:
            for data in buffered_data:
                #print "Sending %s.\n" % data
                self.client.send("%s" % data)
            return True
        except zmq.error.ContextTerminated:
            pass
        except:
            exc_type, exc_value, exc_tb = sys.exc_info()
            if exc_value in ['Interrupted system call', 'Socket operation on non-socket']:
                return False
            self.logger.error("Could not add events to zmq. Exception: %s, Error: %s." % (exc_type, exc_value))
            return False

    def handleEvent(self, event):
        if self.format:
            publish_data = mapDynamicValue(self.format, event)
        else:
            publish_data = msgpack.packb(event)
        if self.topic:
             publish_data = "%s %s" % (self.topic, publish_data)
        self.buffer.append(publish_data)
        yield None

    def shutDown(self):
        try:
            self.buffer.flush()
        except:
            pass
        try:
            self.client.close()
            self.zmq_context.term()
        except AttributeError:
            pass
        # Call parent shutDown method.
        BaseThreadedModule.shutDown(self)



# -*- coding: utf-8 -*-
import threading
import BaseModule


# Conditional imports for python2/3
try:
    import Queue as queue
except ImportError:
    import queue

class BaseThreadedModule(BaseModule.BaseModule, threading.Thread):
    """
    Base class for all lumbermill modules. In most cases this is the class to inherit from when implementing a new module.
    It will only be started as thread when necessary. This depends on the configuration and how the modules are
    combined.

    If you happen to override one of the methods defined here, be sure to know what you
    are doing ;) You have been warned...

    Configuration template:

    - module: SomeModuleName
       id:                               # <default: ""; type: string; is: optional>
       filter:                           # <default: None; type: None||string; is: optional>
       add_fields:                       # <default: {}; type: dict; is: optional>
       delete_fields:                    # <default: []; type: list; is: optional>
       event_type:                       # <default: None; type: None||string; is: optional>
       log_level:                        # <default: 'info'; type: string; values: ['info', 'warn', 'error', 'critical', 'fatal', 'debug']; is: optional>
       queue_size:                       # <default: 20; type: integer; is: optional>
       receivers:
        - ModuleName
        - ModuleAlias
    """

    def __init__(self, gp):
        BaseModule.BaseModule.__init__(self, gp)
        threading.Thread.__init__(self)
        self.input_queue = False
        self.alive = True
        self.daemon = True

    def setInputQueue(self, queue):
        self.input_queue = queue

    def getInputQueue(self):
        return self.input_queue

    def pollQueue(self, block=True, timeout=None):
        try:
            for event in self.input_queue.get(block, timeout):
                yield event
        except (KeyboardInterrupt, SystemExit, ValueError, OSError):
            # Keyboard interrupt is catched in GambolPuttys main run method.
            # This will take care to shutdown all running modules.
            pass

    def run(self):
        # Module will only be run as thread if an input_queue exists. This will depend on the actual configuration.
        # This behaviour can be overwritten by implementing a custom run method.
        if not self.input_queue:
            return False
        if not self.receivers:
            # Only issue warning for those modules that are expected to have receivers.
            # TODO: A better solution should be implemented...
            if self.module_type not in ['stand_alone', 'output']:
                self.logger.error("Shutting down module %s since no receivers are set." % (self.__class__.__name__))
                return
        while self.alive:
            for event in self.pollQueue():
                if not event:
                    continue
                self.receiveEvent(event)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import getopt
import logging.config
import multiprocessing
import os
import signal
import sys
import time
import importlib
from collections import OrderedDict

import tornado.ioloop
import yaml

# Make sure we are called as module. Otherwise imports will fail.
#import inspect
#enty_point = inspect.stack()[-1][3]
#if enty_point != '_run_module_as_main':
#    print('This file needs to be called as module.')
#    print('Usage: %s -m %s -c <path/to/config.conf>' % (sys.executable, os.path.splitext(sys.argv[0])[0]))
#    sys.exit()

from lumbermill.constants import MSGPACK_AVAILABLE, ZMQ_AVAILABLE, LOGLEVEL_STRING_TO_LOGLEVEL_INT
from lumbermill.utils.misc import TimedFunctionManager, coloredConsoleLogging, restartMainProcess
from lumbermill.utils.Buffers import BufferedQueue, ZeroMqMpQueue
from lumbermill.utils.DictUtils import mergeNestedDicts
from lumbermill.utils.ConfigurationValidator import ConfigurationValidator

# Conditional imports for python2/3
try:
    import Queue
except ImportError:
    import queue as Queue

module_dirs = ['input',
               'parser',
               'modifier',
               'misc',
               'output',
               'webserver',
               'cluster']

# Expand the include path to our libs and modules.
# TODO: Check for problems with similar named modules in different module directories.
pathname = os.path.abspath(__file__)
pathname = pathname[:pathname.rfind("/")]
[sys.path.append(pathname + "/" + mod_dir) for mod_dir in module_dirs]

class LumberMill():
    """
    A stream parser with configurable modules and message paths.

    LumberMill helps to parse text based streams by providing a framework of modules.
    These modules can be combined via a simple configuration in any way you like. Have
    a look at the example config lumbermill.conf.example in the conf folder.

    This is the main class that reads the configuration, includes the needed modules
    and connects them as configured.
    """

    def __init__(self, path_to_config_file):
        self.path_to_config_file = path_to_config_file
        self.alive = False
        self.child_processes = []
        self.main_process_pid = os.getpid()
        self.modules = OrderedDict()
        self.global_configuration = {'workers': multiprocessing.cpu_count() - 1,
                                     'queue_size': 20,
                                     'queue_buffer_size': 50,
                                     'logging': {'level': 'info',
                                                 'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                                                 'filename': None,
                                                 'filemode': 'w'}}
        logging.basicConfig(handlers=logging.StreamHandler())
        self.logger = logging.getLogger(self.__class__.__name__)
        success = self.setConfiguration(self.readConfiguration(self.path_to_config_file), merge=False)
        if not success:
            self.shutDown()

    def produceQueue(self, queue_type='simple', queue_max_size=20, queue_buffer_size=1):
        """Returns a queue with queue_max_size"""
        queue = None
        if queue_type == 'simple':
            queue =  BufferedQueue(queue=Queue.Queue(queue_max_size), buffersize=queue_buffer_size)
        if queue_type == 'multiprocess':
            # At the moment I ran into a problem with zmq.
            # This problem causes the performance to be comparable with the normal python multiprocessing.Queue.
            # To make things worse, the load balancing between multiple workers is better when using multiprocessing.Queue.
            # Update 28.04.2015: With pypy-2.5 and normal python, the load balancing problem seems to be gone. ZMQ is
            # still a bit faster (ca. ~15%).
            # TODO: Analyze this problem more thoroughly.
            if False and ZMQ_AVAILABLE and MSGPACK_AVAILABLE:
                  queue = ZeroMqMpQueue(queue_max_size)
            else:
                queue = multiprocessing.Queue(queue_max_size)
            queue = BufferedQueue(queue=queue, buffersize=queue_buffer_size)
        if not queue:
            self.logger.error("Could not produce requested queue %s." % (queue_type))
            self.shutDown()
        return queue

    def readConfiguration(self, path_to_config_file):
        """Loads and parses the configuration"""
        try:
            with open(path_to_config_file, "r") as configuration_file:
                self.raw_conf_file = configuration_file.read()
            configuration = yaml.load(self.raw_conf_file)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not read config file %s. Exception: %s, Error: %s." % (path_to_config_file, etype, evalue))
            usage()
            self.shutDown()
        return configuration

    def getConfigurationFilePath(self):
        return self.path_to_config_file

    def getConfiguration(self):
        return self.configuration

    def getRawConfiguration(self):
        return self.raw_conf_file

    def getWorkerCount(self):
        return self.global_configuration['workers']

    def setConfiguration(self, configuration, merge=True):
        configuration_errors = ConfigurationValidator().validateConfiguration(configuration)
        for configuration_error in configuration_errors:
            self.logger.error("Could not set configuration due to configuration errors.")
            self.logger.error(configuration_error)
            return False
        if merge:
            # If merge is true keep currently configured modules and only merge new ones.
            for module_info in configuration:
                if module_info not in self.configuration:
                    self.configuration.append(module_info)
        else:
            self.configuration = configuration
        return True

    def configureGlobal(self):
        """Merge custom global configuration values to default settings."""
        for idx, configuration in enumerate(self.configuration):
            if 'Global' in configuration:
                self.global_configuration = mergeNestedDicts(self.global_configuration, configuration['Global'])
                self.configuration.pop(idx)

    def configureLogging(self):
        # Reinit logger configuration.
        if self.global_configuration['logging']['level'].lower() not in LOGLEVEL_STRING_TO_LOGLEVEL_INT:
            print("Loglevel unknown.")
            self.shutDown()
        # Remove all handlers associated with the root logger object.
        for handler in logging.root.handlers[:]:
            logging.root.removeHandler(handler)
        log_level = LOGLEVEL_STRING_TO_LOGLEVEL_INT[self.global_configuration['logging']['level'].lower()]
        logging.basicConfig(level=log_level,
                                format=self.global_configuration['logging']['format'],
                                filename=self.global_configuration['logging']['filename'],
                                filemode=self.global_configuration['logging']['filemode'])
        if not self.global_configuration['logging']['filename']:
            logging.StreamHandler.emit = coloredConsoleLogging(logging.StreamHandler.emit)
        self.logger = logging.getLogger(self.__class__.__name__)

    def initModule(self, module_name):
        """ Initalize a module."""
        self.logger.debug("Initializing module %s." % (module_name))
        instance = None
        #module = importlib.import_module(module_name, 'LumberMill')
        module = __import__(module_name, globals(), locals(), module_name, -1)
        module_class = getattr(module, module_name)
        instance = module_class(self)
        """
        try:
            module = importlib.import_module(module_name, 'LumberMill')
            #module = __import__(module_name, globals(), locals(), class_name, -1)
            module_class = getattr(module, module_name)
            instance = module_class(self)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.error("Could not init module %s. Exception: %s, Error: %s." % (module_name, etype, evalue))
            self.shutDown()
        """
        return instance

    def initModulesFromConfig(self):
        """ Initalize all modules from the current config."""
        # Init modules as defined in config
        for idx, module_info in enumerate(self.configuration):
            module_config = {}
            module_id = None
            if isinstance(module_info, dict):
                module_class_name = module_info.keys()[0]
                module_config = module_info[module_class_name]
                # Set module id. If the id field was used in configuration use it else use class name of module.
                try:
                    module_id = module_class_name if 'id' not in module_config else module_config['id']
                except:
                    etype, evalue, etb = sys.exc_info()
                    self.logger.error("Error in configuration file for module %s. Exception: %s, Error: %s. Please check configuration." % (module_class_name, etype, evalue))
                    self.shutDown()
            else:
                module_id = module_class_name = module_info
            counter = 1
            # Modules ids have to be unique. Add a counter like "_1" to id if it is already used.
            while module_id in self.modules:
                tmp_mod_name = module_id.split("_", 1)[0]
                module_id = "%s_%s" % (tmp_mod_name, counter)
                counter += 1
            # Ignore some reserved module names. At the moment this is just the Global keyword.
            if module_id in ['Global']:
                continue
            module_instances = []
            module_instance = self.initModule(module_class_name)
            module_instances.append(module_instance)
            self.modules[module_id] = {'idx': idx,
                                       'instances': module_instances,
                                       'type': module_instance.module_type,
                                       'configuration': module_config}

    def setDefaultReceivers(self):
        """
        Add default receivers if none are provided by configuration.

        To make configuration less complicated, a module does not need to provide a receivers setting.
        if receivers is not set, events are send to the next module in the configuration by default.
        This method takes care of setting the default receivers settings.
        """
        # Iterate over all configured modules ordered as they appear in the current configuration.
        for module_name, module_info in self.modules.items():
            # If receivers is configured we can skip to next module.
            if 'receivers' in module_info['configuration']:
                continue
            # Some module types do not have any receivers.
            if module_info['type'] in ['stand_alone', 'output']:
                module_info['configuration']['receivers'] = []
                continue
            # Break on last module since it can have no following receivers.
            if module_info['idx'] == len(self.modules) - 1:
                break
            # Set receiver to next module in config if no receivers were set.
            # Get next module in configuration.
            try:
                receiver_module_name = [nxt_mod_name for nxt_mod_name, nxt_mod_info in self.modules.items() if nxt_mod_info['idx'] == module_info['idx'] + 1][0]
            except:
                # Something is wrong with the configuration. Tell user.
                etype, evalue, etb = sys.exc_info()
                self.logger.error("Error in configuration for module %s. Exception: %s, Error: %s. Please check configuration." % (module_name, etype, evalue))
                self.shutDown()
            # Some modules are not allowed to act as receivers. Exit if configuration is faulty.
            if self.modules[receiver_module_name]['type'] in ['stand_alone', 'input']:
                self.logger.error("Error in configuration. %s not allowed to act as receiver for events from %s. %s modules are not allowed as receivers. Please check configuration." % (receiver_module_name, module_name, self.modules[receiver_module_name]['type']))
                self.shutDown()
            module_info['configuration']['receivers'] = [receiver_module_name]

    def configureModules(self):
        """Call configuration method of module."""
        for module_name, module_info in sorted(self.modules.items(), key=lambda x: x[1]['idx']):
            for module_instance in module_info['instances']:
                module_instance.configure(module_info['configuration'])

    def initEventStream(self):
        """
        Connect all modules.

        The configuration allows to connect the modules via the <receivers> parameter.
        As different types of modules exists (Base, Threaded, MultiProcess), connecting one modules output
        with its receivers input can be either direct or via a queue.
        TODO: To prevent loops a sanity check should be performed after all modules have been connected.
        """
        queues = {}
        # Iterate over all configured modules.
        for module_name, module_info in self.modules.items():
            sender_instance = module_info['instances'][0]
            for receiver_data in sender_instance.getConfigurationValue('receivers'):
                if not receiver_data:
                    break
                if isinstance(receiver_data, dict):
                    receiver_name, _ = iter(receiver_data.items()).next()
                else:
                    receiver_name = receiver_data
                if receiver_name not in self.modules:
                    self.logger.error("Could not add %s as receiver for %s. Module not found." % (receiver_name, module_name))
                    self.shutDown()
                # If we run multiprocessed and the module is not capable of running parallel, this module will only be
                # started in the main process. Connect the module via a queue to all receivers that can run forked.
                for receiver_instance in self.modules[receiver_name]['instances']:
                    if (self.global_configuration['workers'] > 1 and sender_instance.can_run_forked != receiver_instance.can_run_forked):
                        try:
                            queue = queues[receiver_name]
                        except KeyError:
                            queue = self.produceQueue('multiprocess', self.global_configuration['queue_size'], self.global_configuration['queue_buffer_size'])
                            queues[receiver_name] = queue
                        receiver_instance.setInputQueue(queue)
                # Add the receiver to senders. If a corresponding queue exist, use this else use the normal mod instance.
                for instance in module_info['instances']:
                    if receiver_name in queues:
                        self.logger.debug("%s will send its output to %s via a queue." % (module_name, receiver_name))
                        instance.addReceiver(receiver_name, queues[receiver_name])
                    else:
                        self.logger.debug("%s will send its output directly to %s." % (module_name, receiver_name))
                        instance.addReceiver(receiver_name, receiver_instance)

    def getModuleInfoById(self, module_id, silent=True):
        """
        Get a module by its id.

        Some modules need access to other modules. This method will return the module referenced by its id.
        """
        try:
            return self.modules[module_id]
        except KeyError:
            if not silent:
                self.logger.error("Get module by id %s failed. No such module." % (module_id))
            return None

    def initModulesAfterFork(self):
        """
        All modules are completely configured, call modules initAfterFork method.
        initAfterFork is used to (re)init modules after a process fork.
        BufferedQueue i.e. uses a thread to flush its buffer in given intervals.
        The thread will not survive a fork of the main process. So we need to start this
        after the fork was executed.
        """
        for module_name, module_info in sorted(self.modules.items(), key=lambda x: x[1]['idx']):
            for instance in module_info['instances']:
                if instance.can_run_forked:
                    instance.initAfterFork()
                #else:
                #    print("Not calling initAfterFork on %s." % module_name)

    def runModules(self):
        """
        Start the configured modules if they poll queues.
        """
        # All modules are completely configured, call modules run method if it exists.
        for module_name, module_info in sorted(self.modules.items(), key=lambda x: x[1]['idx']):
            if self.is_master():
                start_message = "%s - %s" % (module_name, module_info['instances'][0].getStartMessage())
                self.logger.info(start_message)
            for instance in module_info['instances']:
                # Some modules, mostly input modules, use unique io devices, that can not be polled from multiple
                # threads/processes. These modules will only be started once from the master process and the output
                # will be send via queue to the other processes.
                if not self.is_master() and not instance.can_run_forked:
                    continue
                # The default 'start' method of threading.Thread/mp will call the 'run' method of the module.
                # The module itself can then decide if it wants to be run as thread. If not, it has to return False to let Gambolputty know.
                if getattr(instance, "start", None): # and (instance.getInputQueue() or instance.module_type in ['stand_alone', 'input'])
                    started = instance.start()
                    if started:
                        self.logger.debug("Starting module %s as thread via %s.start()." % (module_name, module_name))
                    continue

    def getAllQueues(self):
        """ Get all configured queues to check for pending events. """
        module_queues = {}
        for module_name, module_info in self.modules.items():
            instance = module_info['instances'][0]
            if not hasattr(instance, 'getInputQueue') or not instance.getInputQueue():
                continue
            module_queues[module_name] = instance.getInputQueue()
        return module_queues

    def is_master(self):
        return os.getpid() == self.main_process_pid

    def configTest(self):
        self.logger.info("Running configuration test for %s." % self.path_to_config_file)
        self.configureGlobal()
        self.configureLogging()
        self.initModulesFromConfig()
        self.setDefaultReceivers()
        self.configureModules()
        self.initEventStream()
        self.logger.info("Done...")
        return

    def start(self):
        self.configureGlobal()
        self.configureLogging()
        self.initModulesFromConfig()
        self.setDefaultReceivers()
        self.configureModules()
        self.initEventStream()
        self.runWorkers()

    def runWorkers(self):
        for i in range(1, self.global_configuration['workers']):
            worker = multiprocessing.Process(target=self.run)
            worker.start()
            self.child_processes.append(worker)
        self.run()

    def run(self):
        # Catch Keyboard interrupt here. Catching the signal seems
        # to be more reliable then using try/except when running
        # multiple processes under pypy.
        # Register SIGINT to call shutDown for all processes.
        signal.signal(signal.SIGINT, self.shutDown)
        if self.is_master():
            # Register SIGALARM only for master process. This will take care to kill all subprocesses.
            signal.signal(signal.SIGALRM, self.restart)
        self.alive = True
        self.initModulesAfterFork()
        self.runModules()
        if self.is_master():
            self.logger.info("LumberMill started with %s processes(%s)." % (len(self.child_processes) + 1, os.getpid()))
        tornado.ioloop.IOLoop.instance().start()

    def restart(self, signum=False, frame=False):
        for worker in list(self.child_processes):
            os.kill(worker.pid, signal.SIGINT)
            worker.join()
            self.child_processes.remove(worker)
        self.logger.info("Restarting LumberMill.")
        self.shutDown()
        time.sleep(5)
        restartMainProcess()

    def shutDown(self, signum=False, frame=False):
        if self.is_master():
            self.logger.info("Shutting down LumberMill.")
            # Send SIGINT to workers for good measure.
            for worker in list(self.child_processes):
                os.kill(worker.pid, signal.SIGINT)
        if not self.alive:
            sys.exit(0)
        self.alive = False
        self.shutDownModules()
        TimedFunctionManager.stopTimedFunctions()
        tornado.ioloop.IOLoop.instance().stop()
        if self.is_master():
            self.logger.info("Shutdown complete.")

    def shutDownModules(self):
        # Shutdown all input modules.
        for module_name, module_info in self.modules.items():
            for instance in module_info['instances']:
                if instance.module_type == "input":
                    instance.shutDown()
        # Get all configured queues to check for pending events.
        module_queues = self.getAllQueues()
        if len(module_queues) > 0:
            wait_loops = 0
            while wait_loops < 5:
                wait_loops += 1
                events_in_queues = 0
                for module_name, queue in module_queues.items():
                    events_in_queues += queue.qsize()
                if events_in_queues > 0:
                    # Give remaining queued events some time to finish.
                    if self.is_master():
                        self.logger.info("%s event(s) still in flight. Waiting %s secs. Press ctrl+c again to exit directly." % (events_in_queues, (.5 * wait_loops)))
                    time.sleep(.5 * wait_loops)
                    continue
                break
        # Shutdown all other modules.
        for module_name, module_info in self.modules.items():
            for instance in module_info['instances']:
                if instance.module_type != "input":
                    instance.shutDown()

def usage():
    print('Usage: ' + sys.argv[0] + ' -c <path/to/config.conf> --configtest')

def main():
    path_to_config_file = ""
    run_configtest = False
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hc:", ["help", "configtest", "conf="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-c", "--conf"):
            path_to_config_file = arg
        elif opt in ("--configtest"):
            run_configtest = True
    gp = LumberMill(path_to_config_file)
    if run_configtest:
        gp.configTest()
    else:
        gp.start()

if __name__ == '__main__':
    main()






# -*- coding: utf-8 -*-
import sys
import types

import msgpack

from lumbermill.BaseThreadedModule import BaseThreadedModule
from lumbermill.utils.Decorators import ModuleDocstringParser


@ModuleDocstringParser
class MsgPackParser(BaseThreadedModule):
    """
    Decode:
     It will parse the msgpack data and create or replace fields in the internal data dictionary with
     the corresponding json fields.
    Encode:
     Encode selected fields or all to msgpack format.

    Configuration template:

    - MsgPackParser:
       action:                          # <default: 'decode'; type: string; values: ['decode','encode']; is: optional>
       mode:                            # <default: 'line'; type: string; values: ['line','stream']; is: optional>
       source_fields:                   # <default: 'data'; type: string||list; is: optional>
       target_field:                    # <default: None; type: None||string; is: optional>
       keep_original:                   # <default: False; type: boolean; is: optional>
       receivers:
        - NextModule
    """

    module_type = "parser"
    """Set module type"""

    def configure(self, configuration):
        # Call parent configure method
        BaseThreadedModule.configure(self, configuration)
        self.source_fields = self.getConfigurationValue('source_fields')
        # Allow single string as well.
        if isinstance(self.source_fields, types.StringTypes):
            self.source_fields = [self.source_fields]
        self.target_field = self.getConfigurationValue('target_field')
        self.drop_original = not self.getConfigurationValue('keep_original')
        if self.getConfigurationValue('action') == 'decode':
            if self.getConfigurationValue('mode') == 'line':
                self.handleEvent = self.decodeEventLine
            else:
                self.unpacker = msgpack.Unpacker()
                self.handleEvent = self.decodeEventStream
        else:
            self.handleEvent = self.encodeEvent

    def decodeEventStream(self, event):
        for source_field in self.source_fields:
            if source_field not in event:
                continue
            self.unpacker.feed(event[source_field])
            # If decoded data contains more than one event, we need to clone all events but the first one.
            # Otherwise we will have multiple events with the same event_id.
            # KeyDotNotationDict.copy method will take care of creating a new event id.
            for events_count, decoded_data in enumerate(self.unpacker):
                if events_count > 1:
                    event = event.copy()
                if not isinstance(decoded_data, dict):
                    continue
                if self.drop_original:
                    event.pop(source_field, None)
                if self.target_field:
                    event.update({self.target_field: decoded_data})
                else:
                    try:
                        event.update(decoded_data)
                    except:
                        etype, evalue, etb = sys.exc_info()
                        self.logger.warning("Could not update event with msgpack data: %s. Exception: %s, Error: %s." % (decoded_data, etype, evalue))
                yield event

    def decodeEventLine(self, event):
        for source_field in self.source_fields:
            if source_field not in event:
                continue
            try:
                decoded_data = msgpack.unpackb(event[source_field])
            except:
                etype, evalue, etb = sys.exc_info()
                self.logger.warning("Could not parse msgpack event data: %s. Exception: %s, Error: %s." % (event[source_field], etype, evalue))
                continue
            if self.drop_original:
                event.pop(source_field, None)
            if self.target_field:
                event.update({self.target_field: decoded_data})
            else:
                try:
                    event.update(decoded_data)
                except:
                    etype, evalue, etb = sys.exc_info()
                    self.logger.warning("Could not update event with msgpack data: %s. Exception: %s, Error: %s." % (decoded_data, etype, evalue))
        yield event

    def encodeEvent(self, event):
        if self.source_fields == ['all']:
            encode_data = event
        else:
            encode_data = []
            for source_field in self.source_fields:
                if source_field not in event:
                    continue
                encode_data.append({source_field: event[source_field]})
                if self.drop_original:
                    event.pop(source_field, None)
        try:
            encode_data = msgpack.packb(encode_data)
        except:
            etype, evalue, etb = sys.exc_info()
            self.logger.warning("Could not msgpack encode event data: %s. Exception: %s, Error: %s." % (event, etype, evalue))
            yield event
            return
        if self.source_fields == ['all']:
            event = encode_data
        else:
            event.update({self.target_field: encode_data})
        yield event